# Generated by Django 3.1.2 on 2020-10-31 16:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0010_viergewinntgame'),
    ]

    operations = [
        migrations.AlterField(
            model_name='viergewinntgame',
            name='player_2_id',
            field=models.CharField(default=None, max_length=32, null=True, verbose_name='Player 2 ID'),
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-22 16:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0005_amongusgame_text_message_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='amongusgame',
            name='p_black_exists',
            field=models.BooleanField(default=False, verbose_name='Black exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_blue_exists',
            field=models.BooleanField(default=False, verbose_name='Blue exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_brown_exists',
            field=models.BooleanField(default=False, verbose_name='Brown exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_cyan_exists',
            field=models.BooleanField(default=False, verbose_name='Cyan exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_green_exists',
            field=models.BooleanField(default=False, verbose_name='Green exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_lime_exists',
            field=models.BooleanField(default=False, verbose_name='Lime exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_orange_exists',
            field=models.BooleanField(default=False, verbose_name='Orange exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_pink_exists',
            field=models.BooleanField(default=False, verbose_name='Pink exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_purple_exists',
            field=models.BooleanField(default=False, verbose_name='Purple exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_red_exists',
            field=models.BooleanField(default=False, verbose_name='Red exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_white_exists',
            field=models.BooleanField(default=False, verbose_name='White exists'),
        ),
        migrations.AlterField(
            model_name='amongusgame',
            name='p_yellow_exists',
            field=models.BooleanField(default=False, verbose_name='Yellow exists'),
        ),
    ]



from django.core.management.base import BaseCommand, CommandError

import os

from discordbot.bot import run

class Command(BaseCommand):
    help = 'Runs the Discord bot'

    def add_arguments(self, parser):
        parser.add_argument('--token', help="Discords Bot Token", type=str)

    def handle(self, *args, **options):
        self.stdout.write(self.style.SUCCESS("[Bot] - Bot wird gestartet..."))

        if "token" in options and options["token"]:
            run(options["token"])
        elif 'DISCORDBOT_TOKEN' in os.environ:
            run(os.environ.get('DISCORDBOT_TOKEN'))
        elif 'DISCORD_BOTTOKEN' in os.environ:
            run(os.environ.get('DISCORD_BOTTOKEN'))
        else:
            print("[Bot] - No TOKEN found!")

        self.stdout.write(self.style.SUCCESS("[Bot] - Bot wurde beendet!"))



# Generated by Django 3.1.2 on 2020-10-23 19:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0006_auto_20201022_1804'),
    ]

    operations = [
        migrations.AddField(
            model_name='amongusgame',
            name='p_black_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Black's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_blue_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Blue's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_brown_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Brown's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_cyan_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Cyan's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_green_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Green's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_lime_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Lime's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_orange_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Orange's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_pink_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Pink's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_purple_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Purple's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_red_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Red's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_white_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="White's Discord ID"),
        ),
        migrations.AddField(
            model_name='amongusgame',
            name='p_yellow_userid',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name="Yellow's Discord ID"),
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-21 21:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0019_auto_20201115_1844'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='server',
            options={'verbose_name': 'Guild', 'verbose_name_plural': 'Guilds'},
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-10 18:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0014_auto_20201110_1826'),
    ]

    operations = [
        migrations.AddField(
            model_name='notifiersub',
            name='name',
            field=models.CharField(default='Unbenannte Mitteilung', max_length=50, verbose_name='Name'),
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-14 19:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0017_audiosource'),
    ]

    operations = [
        migrations.AlterField(
            model_name='audiosource',
            name='url_source',
            field=models.TextField(verbose_name='Url (Source)'),
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-23 20:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0008_amongusgame_last_tracking_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='amongusgame',
            name='tracker_connected',
        ),
    ]



from discord.ext import commands
from discord import Embed, User, TextChannel, utils
from datetime import datetime as d
import typing

NUMBER_EMOJIS = ["1️⃣", "2️⃣", "3️⃣", "4️⃣", "5️⃣", "6️⃣", "7️⃣", "8️⃣", "9️⃣", "🔟"]
BOOL_EMOJIS = ["✅", "❌"]

class Polls(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0xff8c00

    @commands.command(
        brief="Erstelle eine Umfrage",
        aliases=["umfrage"],
        usage="<Frage> [1-10 Antworten]",
        help="Eine Antwort pro Zeile. Wenn keine Antworten angegeben werden, wird standardmässig Ja und Nein verwendet."
    )
    async def poll(self, ctx, *, msg):
        answers = msg.split("\n")
        question = answers.pop(0)
        if len(answers) == 0:
            msg = await ctx.sendEmbed(
                title=question,
                description=f"{BOOL_EMOJIS[0]} - Ja\n{BOOL_EMOJIS[1]} - Nein",
            )
            for e in BOOL_EMOJIS:
                await msg.add_reaction(e)
        else:
            description = "\n".join([f"{NUMBER_EMOJIS[i]} - {a}" for i, a in enumerate(answers[:10])])
            msg = await ctx.sendEmbed(
                title=question,
                description=description,
            )
            for e in NUMBER_EMOJIS[:len(answers)]:
                await msg.add_reaction(e)

def setup(bot):
    bot.add_cog(Polls(bot))



from discord.ext import commands
from discord import utils
from datetime import datetime as d
import typing
from discordbot.botmodules import converters

class Converters(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0xffffff

    @commands.command(
        brief="Wandle Morsecode um",
        aliases=["mors","morsecode"],
        help="Benutze /morse <Text> und erhalte den Text in Morsecode oder umgekehrt",
        usage="<Morsecode | Nachricht>"
        )
    async def morse(self, ctx, text: str, *args):
        message = " ".join((text,)+args)
        if message.replace("-", "").replace("_", "").replace(".", "").replace("|", "").replace(" ", "") == "":
            text = converters.morse_decrypt(message.replace("_","-").replace(" | ", "  "))
            morse = message
        else:
            text = message
            morse = converters.morse_encrypt(message)
        await ctx.sendEmbed(title="Morsecode", fields=[("Morsecode", morse.replace("  "," | ")),("Text", text)])


def setup(bot):
    bot.add_cog(Converters(bot))



from discord.ext import commands
import requests, datetime, base64, os

from discordbot.errors import ErrorMessage

class Minecraft():
    @classmethod
    def getProfile(self, NAME:str):
        r = requests.get('https://api.mojang.com/users/profiles/minecraft/'+NAME)
        if not r.status_code == 204:
            return r.json()
        else:
            raise commands.BadArgument(message="Spieler wurde nicht gefunden!")

    @classmethod
    def getProfiles(self, UUID:str):
        r = requests.get('https://api.mojang.com/user/profiles/'+str(UUID)+'/names')
        if not r.status_code == 204:
            data = r.json()
            for i in data:
                if "changedToAt" in i:
                    i["changedToAt"] = datetime.datetime.fromtimestamp(int(i["changedToAt"])/1000)
            return data
        else:
            raise commands.BadArgument(message="UUID wurde nicht gefunden!")

    @classmethod
    def getSkin(self, UUID:str):
        r = requests.get('https://sessionserver.mojang.com/session/minecraft/profile/'+str(UUID))
        if not r.status_code == 204:
            data = r.json()
            if not "error" in data:
                ppty = data["properties"][0]
                base64_message = ppty["value"]
                base64_bytes = base64_message.encode('ascii')
                message_bytes = base64.b64decode(base64_bytes)
                message = message_bytes.decode('ascii')
                dictmessage = eval(message)
                if not dictmessage["textures"] == {}:
                    skinurl = dictmessage["textures"]["SKIN"]["url"]
                    data["skin"] = skinurl
                else:
                    data["skin"] = None
                data.pop("properties")
                return data
            else:
                raise commands.BadArgument(message="Abfrage für einen Skin kann pro UUID maximal ein Mal pro Minute erfolgen!")
        else:
            raise commands.BadArgument(message="UUID wurde nicht gefunden!")


class Fortnite():
    @classmethod
    def __get_headers(self):
        key = os.environ.get("TRNAPIKEY", None)
        if key is None:
            raise ErrorMessage(message="Der Fortnite-Befehl ist leider deaktiviert (nicht konfiguriert)!") 
        else:
            return {'TRN-Api-Key': key}

    @classmethod
    def __get_json(self, url, **kwargs):
        try:
            r = requests.get(url, headers=self.__get_headers())
            j = r.json(**kwargs)
            return j
        except (KeyError, ValueError) as e:
            print("[Fortnite API] - Error:", e)

    @classmethod
    def getStore(self):
        return self.__get_json('https://api.fortnitetracker.com/v1/store')

    @classmethod
    def getChallenges(self):
        return self.__get_json('https://api.fortnitetracker.com/v1/challenges')["items"]

    @classmethod
    def getStats(self, platform:str, playername:str):
        if platform.lower() in ["kbm","gamepad","touch"]:
            return self.__get_json(("https://api.fortnitetracker.com/v1/profile/%s/%s" % (platform.lower(), playername)))
        else:
            raise commands.BadArgument("Die Plattform '"+platform+"' existiert nicht! Benutze kbm, gamepad oder touch!")



# Generated by Django 3.1.3 on 2020-11-14 19:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0016_auto_20201113_2031'),
    ]

    operations = [
        migrations.CreateModel(
            name='AudioSource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url_source', models.URLField(unique=True, verbose_name='Url (Source)')),
                ('url_watch', models.URLField(blank=True, default='', verbose_name='Url (Watch)')),
                ('url_thumbnail', models.URLField(blank=True, default='', verbose_name='Url (Thumbnail)')),
                ('title', models.CharField(max_length=256, verbose_name='Title')),
                ('description', models.TextField(max_length=2048, verbose_name='Description')),
                ('duration', models.IntegerField(blank=True, default=0, verbose_name='Duration')),
                ('uploader_name', models.CharField(blank=True, default='', max_length=256, verbose_name='Uploader name')),
                ('uploader_url', models.URLField(blank=True, default='', verbose_name='Uploader Url')),
            ],
            options={
                'verbose_name': 'Audio source',
                'verbose_name_plural': 'Audio sources',
            },
        ),
    ]



from discord.ext import commands
from discord import utils
import typing

#from discordbot.config import 

class PRESET(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0xffffff

    @commands.command(
        brief="",
        description="",
        aliases=[],
        help="",
        usage=""
        )
    async def preset(self, ctx):
        pass

def setup(bot):
    bot.add_cog(PRESET(bot))



from discord.ext import commands
from discord import Embed, Message, Color
import random

par = "//"
opt = "/!/"

class EmbedGenerator(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0x34b7eb

    @commands.has_permissions(manage_messages = True)
    @commands.guild_only()
    @commands.command(
        name='createembed',
        brief='Erstelle einen Embed',
        description='Erstelle einen Embed im Chat!',
        aliases=["cemb","embedcreate"],
        help="Benutze /createembed für Erklärungen.",
        usage="<Titel> [Argumente]"
    )
    async def createembed(self, ctx):
        text = " ".join(ctx.message.content.split(ctx.invoked_with)[1:])
        if not text:
            await ctx.sendEmbed(title="Embed-Creator", description=f"""
Verwendung des Chat-Embed-Generators:
```/createembed <Titel>
[Beschreibung]
[Parameter1]
[Parameter2]
[...]
```
Parameter brauchen je eine Zeile und beginnen immer mit `{par}`.
Optionen werden immer mit `{opt}` getrennt.

Verfügbare Parameter:""", inline=False, fields=[
        ("Feld (bis zu 25)",f"{par}field{opt}<Titel>{opt}<Inhalt>{opt}[Inline]"),
        ("Footer",          f"{par}footer{opt}<Titel>{opt}[Bild-URL]"),
        ("Author",          f"{par}author{opt}<Name>{opt}[Link]"),
        ("Thumbnail",       f"{par}thumbnail{opt}<Bild-URL>"),
        ("Color",           f"{par}color{opt}<RED>{opt}<GREEN>{opt}<BLUE>")
        ])
        else:
            lines = text.split("\n")
            data = {
                "title": lines.pop(0),
                "description": "",
                "footertext": "",
                "footerurl": "",
                "authorname": "",
                "authorurl": "",
                "fields": [],
                "thumbnailurl": "",
                "timestamp": False
            }

            for line in lines:
                if len(line) >= 4 and line.startswith(par):
                    line = line[2::]
                    options = line.split(opt)
                    command = options.pop(0).lower().strip()
                    if command in ["field","f","field"] and len(options) >= 2:
                        data["fields"].append((options[0],options[1]) if (len(options) < 3 or not (options[2].lower() in ["f","false","no"])) else (options[0],options[1],False))
                    elif command in ["footer","foot"] and len(options) >= 1:
                        data["footertext"] = options[0]
                        data["footerurl"] = options[1] if len(options) > 1 else ""
                    elif command in ["author"] and len(options) >= 1:
                        data["authorname"] = options[0]
                        data["authorurl"] = options[1] if len(options) > 1 else ""
                    elif command in ["thumbnailurl", "thumbnail", "thumb"] and len(options) >= 1:
                        data["thumbnailurl"] = options[0]
                    elif command in ["color", "c"] and len(options) >= 3:
                        try:
                            data["color"] = Color.from_rgb(int(options[0]),int(options[1]),int(options[2]))
                        except ValueError:
                            pass
                else:
                    data["description"] += "\n"+line

            await ctx.sendEmbed(**data)

    @commands.command(
        name='getembed',
        brief='Erhalte einen Embed',
        description='Erhalte den Command, um einen bestehenden Embed zu erstellen.',
        aliases=["gemb","getemb"],
        help="Benutze /getembed <NachrichtenID>",
        usage="<NachrichtenID>"
    )
    async def getembed(self, ctx, msg:Message):
        if msg.embeds:
            emb = msg.embeds[0]
            text = "/createembed "+emb.title
            if emb.description:
                text += f"\n{emb.description}"
            if emb.footer:
                text += f"\n{par}footer{opt}{emb.footer.text}{(opt+emb.footer.icon_url) if emb.footer.icon_url else ''}"
            if emb.thumbnail:
                text += f"\n{par}thumbnail{opt}{emb.thumbnail.url}"
            if emb.author:
                text += f"\n{par}author{opt}{emb.author.name}{(opt+emb.author.url) if emb.author.url else ''}"
            if emb.fields:
                for field in emb.fields:
                    text += f"\n{par}field{opt}{field.name}{opt}{field.value}{(opt+'False') if not field.inline else ''}"
            if emb.color:
                text += f"\n{par}color{opt}{str(emb.color.r)}{opt}{str(emb.color.g)}{opt}{str(emb.color.b)}"
            await ctx.sendEmbed(title="Embed Command", description=text)
        else:
            raise commands.BadArgument("Diese Nachricht hat kein Embed!")

def setup(bot):
    bot.add_cog(EmbedGenerator(bot))



from django.contrib import admin
from .models import Server, User, Report, Member, AmongUsGame, AMONGUS_PLAYER_COLORS, VierGewinntGame, NotifierSub

# General

class ServerAdminMemberInline(admin.TabularInline):
    model = Member
    extra = 0

    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

class ServerAdminReportInline(admin.TabularInline):
    model = Report
    fk_name = "server"
    extra = 0
    verbose_name_plural = "Reports auf diesem Server"

    readonly_fields = ("reported_by",)

    def has_change_permission(self, request, obj=None):
        return False

@admin.register(Server)
class ServerAdmin(admin.ModelAdmin):
    fields = ("id","name")
    readonly_fields = ("id","name")

    list_display = ("id","name","reportCount")

    inlines = [ServerAdminReportInline, ServerAdminMemberInline]

    def has_add_permission(self, request, obj=None):
        return False



class UserAdminReportInline(admin.TabularInline):
    model = Report
    fk_name = "user"
    extra = 0
    verbose_name_plural = "Reports gegen diesen Benutzer"

    readonly_fields = ("reported_by",)

    def has_change_permission(self, request, obj=None):
        return False

class UserAdminReportCreatedInline(admin.TabularInline):
    model = Report
    fk_name = "reported_by"
    extra = 0
    verbose_name_plural = "Reports von diesem Benutzer"

    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

class UserAdminServerInline(admin.TabularInline):
    model = Member
    extra = 0
    verbose_name_plural = "Server"

    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

@admin.register(User)
class UserAdmin(admin.ModelAdmin):
    fields = ("id","name")
    readonly_fields = ("id","name")

    list_display = ("id","name","reportCount","createdReportCount")

    inlines = [UserAdminReportInline, UserAdminReportCreatedInline, UserAdminServerInline]

    def has_add_permission(self, request, obj=None):
        return False

# Games

@admin.register(AmongUsGame)
class AmongUsGameAdmin(admin.ModelAdmin):
    list_display = ('id', 'code', 'state_ingame', 'state_meeting', 'creator')
    ordering = ('code', )

    readonly_fields = ("last_tracking_data", "last_edited",)

    fieldsets = [
        ('Info', {'fields': ('code', )}),
        ('Times', {'fields': ('last_tracking_data', 'last_edited')}),
        ('State', {'fields': ('state_ingame', 'state_meeting', )}),
        ('Players', {'fields': 
            tuple((f'p_{c}_name', f'p_{c}_alive', f'p_{c}_exists', f'p_{c}_userid') 
            for c in AMONGUS_PLAYER_COLORS)
        }),
    ]

    def has_add_permission(self, request, obj=None):
        return False

@admin.register(VierGewinntGame)
class VierGewinntAdmin(admin.ModelAdmin):
    list_display = ('id', 'player_1_id', 'player_2_id', 'finished', 'winner_id', 'time_edited', 'time_created')
    ordering = ('id',)

    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    fields = ('get_description',)
    readonly_fields = ('get_description',)

@admin.register(NotifierSub)
class NotifierSubAdmin(admin.ModelAdmin):
    list_display = ('name', 'where_type', 'where_id', 'frequency', 'url', 'must_contain_regex',)
    ordering = ('id',)

    fieldsets = [
        ("",        {"fields": ('name',)}),
        ("Ziel",    {"fields": ('where_type', 'where_id',)}),
        ("Herkunft",{"fields": ('url', 'frequency')}),
        ("Filter",  {"fields": ('must_contain_regex',)})
    ]



# Generated by Django 3.0.4 on 2020-07-01 13:55

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Member',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'verbose_name': 'Mitglied',
                'verbose_name_plural': 'Mitglieder',
            },
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('discordid', models.CharField(max_length=20, primary_key=True, serialize=False, verbose_name='Discord ID')),
                ('name', models.CharField(blank=True, default='', max_length=100, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Benutzer',
                'verbose_name_plural': 'Benutzer',
            },
        ),
        migrations.CreateModel(
            name='Server',
            fields=[
                ('discordid', models.CharField(max_length=20, primary_key=True, serialize=False, verbose_name='Discord ID')),
                ('name', models.CharField(blank=True, default='', max_length=100, verbose_name='Name')),
                ('members', models.ManyToManyField(through='discordbot.Member', to='discordbot.User')),
            ],
            options={
                'verbose_name': 'Server',
                'verbose_name_plural': 'Server',
            },
        ),
        migrations.CreateModel(
            name='Report',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reason', models.CharField(blank=True, default='', max_length=250, verbose_name='Grund')),
                ('timestamp', models.DateTimeField(auto_now_add=True, verbose_name='Zeitpunkt')),
                ('reported_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='created_reports', to='discordbot.User', verbose_name='Gemeldet von')),
                ('server', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reports', to='discordbot.Server')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reports', to='discordbot.User')),
            ],
            options={
                'verbose_name': 'Report',
                'verbose_name_plural': 'Reports',
            },
        ),
        migrations.AddField(
            model_name='member',
            name='server',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='discordbot.Server'),
        ),
        migrations.AddField(
            model_name='member',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='servers', to='discordbot.User'),
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-10 17:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0012_botpermission'),
    ]

    operations = [
        migrations.CreateModel(
            name='NotifierSub',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('where_type', models.CharField(choices=[('channel', 'Kanal'), ('member', 'Mitglied (DM)')], max_length=8)),
                ('where_id', models.CharField(max_length=32, verbose_name='ID 1')),
                ('frequency', models.CharField(choices=[('hour', 'Stündlich'), ('minute', 'Minütlich')], max_length=8, verbose_name='Häufigkeit')),
                ('url', models.URLField(verbose_name='URL')),
                ('must_contain_regex', models.CharField(default='', max_length=32, verbose_name='Muss Regex enthalten')),
                ('last_hash', models.CharField(max_length=32, verbose_name='Letzter Hash')),
            ],
        ),
    ]



import typing
from datetime import datetime

from discord import Activity, ActivityType, Embed, Game, HTTPException, Status, Member, User, TextChannel, VoiceChannel, Role, Invite, Game, Emoji, PartialEmoji, Colour
from discord.ext import commands

from discordbot.botmodules import serverdata, audio
from discordbot.config import EXTENSIONFOLDER, EXTENSIONS, ALL_PREFIXES, MAIN_PREFIXES, DEBUG
from discordbot.utils import chunks

from rich.traceback import install as install_traceback
install_traceback()
from rich.pretty import install as install_pretty
install_pretty()

#

CONVERTERS = {
    Member: commands.MemberConverter,
    User: commands.UserConverter,
    TextChannel: commands.TextChannelConverter,
    VoiceChannel: commands.VoiceChannelConverter,
    Role: commands.RoleConverter,
    Invite: commands.InviteConverter,
    Game: commands.GameConverter,
    Emoji: commands.EmojiConverter,
    PartialEmoji: commands.PartialEmojiConverter,
    Colour: commands.ColourConverter
}

# Own classes

class MyContext(commands.Context):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.database = serverdata.DjangoConnection(self.author, self.guild)
        self.audio = audio.AudioManager(self)

        if self.guild is not None:
            self.data = serverdata.Server.getServer(self.guild.id)

    async def sendEmbed(self, title: str, *args, receiver=None, message: str = "", description: str = "", fields: list = [], **kwargs):
        if len(description) > 2048:
            desc = list(chunks(description, 2042))
            for i in range(len(desc)):
                if i == 0:
                    await (receiver or self).send(message, embed=self.getEmbed(f"{title} ({i+1}/{len(desc)})", *args, description=desc[i]+" [...]", fields=fields, **kwargs))
                elif i == len(desc)-1:
                    return await (receiver or self).send(embed=self.getEmbed(f"{title} ({i+1}/{len(desc)})", *args, description=desc[i], **kwargs))
                else:
                    await (receiver or self).send(embed=self.getEmbed(f"{title} ({i+1}/{len(desc)})", *args, description=desc[i]+" [...]", **kwargs))
        elif len(fields) > 25:
            flds = list(chunks(fields, 25))
            for i in range(len(flds)):
                if i == 0:
                    await (receiver or self).send(message, embed=self.getEmbed(f"{title} ({i+1}/{len(flds)})", *args, description=description, fields=flds[i], **kwargs))
                elif i == len(flds)-1:
                    return await (receiver or self).send(embed=self.getEmbed(f"{title} ({i+1}/{len(flds)})", *args, fields=flds[i], **kwargs))
                else:
                    await (receiver or self).send(embed=self.getEmbed(f"{title} ({i+1}/{len(flds)})", *args, fields=flds[i], **kwargs))
        else:
            return await (receiver or self).send(message, embed=self.getEmbed(title=title, *args, description=description, fields=fields, **kwargs))

    def getEmbed(self, title:str, description:str="", color:int=0x000000, fields:list=[], inline=True, thumbnailurl:str=None, authorurl:str="", authorname:str=None, footertext:str="Angefordert von USER", footerurl:str="AVATARURL", timestamp=False):
        EMBED = Embed(title=title[:256], description=description[:2048], color=color or getattr(self.cog, "color", 0x000000))
        EMBED.set_footer(text=footertext.replace("USER", str(self.author.name+"#"+self.author.discriminator))[:2048], icon_url=footerurl.replace("AVATARURL", str(self.author.avatar_url)))
        
        if timestamp:
            EMBED.timestamp = datetime.utcnow() if timestamp is True else timestamp
        for field in fields[:25]:
            EMBED.add_field(name=field[0][:256], value=(field[1][:1018]+" [...]" if len(field[1]) > 1024 else field[1]), inline=bool(field[2] if len(field) > 2 else inline))
        if thumbnailurl:
            EMBED.set_thumbnail(url=thumbnailurl.strip())
        if authorname:
            if authorurl and ("https://" in authorurl or "http://" in authorurl):
                EMBED.set_author(name=authorname[:256], url=authorurl.strip())
            else:
                EMBED.set_author(name=authorname[:256])
        return EMBED

    async def tick(self, value=True):
        emoji = '\N{WHITE HEAVY CHECK MARK}' if value else '\N{CROSS MARK}'
        try:
            await self.message.add_reaction(emoji)
        except HTTPException:
            pass

    async def send_help(self):
        await self.invoke(self.bot.get_command("help"), self.invoked_with)

    async def invoke_as(self, member, command, *args):
        _command = command.replace("_", " ")
        cmd = self.bot.get_command(_command)
        if cmd is None:
            raise commands.BadArgument(f"Der Befehl `{ _command }` wurde nicht gefunden! \nPS: Benutze im Command bitte kein Prefix!")
        self.message.content = self.prefix+_command+self.message.content.split(command)[1]
        self.message.author = member
        self.author = member
        self.database = type(self.database)(self.author, self.guild)
        annotations = cmd.callback.__annotations__
        annotations.pop("return", None)
        arguments = list(args)
        for i, cls in enumerate(annotations.values()):
            if len(arguments) > i:
                if cls in CONVERTERS:
                    arguments[i] = await CONVERTERS[cls]().convert(self, arguments[i])
                else:
                    arguments[i] = cls(arguments[i])
        await self.invoke(cmd, *arguments)

class MyBot(commands.Bot):
    def __init__(self, **kwargs):
        super().__init__(self.get_command_prefix, **kwargs)

    def get_command_prefix(self, client, message):
        if message.guild:
            prefixes = MAIN_PREFIXES
        else:
            prefixes = ALL_PREFIXES
        return commands.when_mentioned_or(*prefixes)(client, message)
        
    async def get_context(self, message, *, cls=MyContext):
        return await super().get_context(message, cls=cls)

    def getEmbed(self, title: str, description: str = "", color: int = 0x000000, fields: list = [], inline=True, thumbnailurl: str = None, authorurl: str = "", authorname: str = None, footertext: str = None, footerurl: str = None, timestamp=False):
        EMBED = Embed(title=title[:256], description=description[:2048], color=color)
        if footertext:
            if footerurl:
                EMBED.set_footer(text=footertext[:2048], icon_url=footerurl)
            else:
                EMBED.set_footer(text=footertext[:2048])

        if timestamp:
            EMBED.timestamp = datetime.utcnow() if timestamp is True else timestamp
        for field in fields:
            EMBED.add_field(name=field[0][:256], value=(field[1][:1018]+" [...]" if len(field[1]) > 1024 else field[1]), inline=bool(
                field[2] if len(field) > 2 else inline))
        if thumbnailurl:
            EMBED.set_thumbnail(url=thumbnailurl.strip())
        if authorname:
            if authorurl and ("https://" in authorurl or "http://" in authorurl):
                EMBED.set_author(name=authorname[:256], url=authorurl.strip())
            else:
                EMBED.set_author(name=authorname[:256])
        return EMBED


# create Bot

bot = MyBot(
    description='Das ist eine Beschreibung!',
    case_insensitive=True,
    activity=Activity(type=ActivityType.listening, name=(MAIN_PREFIXES[0] if MAIN_PREFIXES else "/")+"help"),
    status=Status.idle,
    help_command=None,
)


@bot.before_invoke
async def before_invoke(ctx):
    await ctx.trigger_typing()


# Events

from discordbot.botevents import setup

setup(bot)

# Start

def run(TOKEN):
    print("[Bot] - Starting with DEBUG="+str(DEBUG))
    bot.run(TOKEN, bot=True, reconnect=True)

if __name__ == "__main__":
    print("[Bot] - You must run this bot via your manage.py file: python3.8 manage.py run-discorbot")



from discord.ext.commands import BadArgument

MORSE_CODE_DICT = { 'A':'.-',       'B':'-...',
                    'C':'-.-.',     'D':'-..',      'E':'.',
                    'F':'..-.',     'G':'--.',      'H':'....',
                    'I':'..',       'J':'.---',     'K':'-.-',
                    'L':'.-..',     'M':'--',       'N':'-.',
                    'O':'---',      'P':'.--.',     'Q':'--.-',
                    'R':'.-.',      'S':'...',      'T':'-',
                    'U':'..-',      'V':'...-',     'W':'.--',
                    'X':'-..-',     'Y':'-.--',     'Z':'--..',
                    'Ä':'.-.-',     'Ö':'---.',     'Ü':'..--',
                    '1':'.----',    '2':'..---',    '3':'...--',
                    '4':'....-',    '5':'.....',    '6':'-....',
                    '7':'--...',    '8':'---..',    '9':'----.',
                    '0':'-----',    ',':'--..--',   '.':'.-.-.-',
                    ':':'---...',   "'":'.----.',   '"':'-.--.-',
                    '?':'..--..',   '/':'-..-.',    '-':'-....-',
                    '@':'.--.-.',   '=':'-...-',    '&':'.-...',
                    '(':'-.--.',    ')':'-.--.-',   '!':'-.-.--',
                    'UNDERSTOOD': '...-.'
                }

MORSE_CODE_DICT_INVERTED = {v: k for k, v in MORSE_CODE_DICT.items()}

# Text zu Morsecode
def morse_encrypt(message):
    message = message.upper().lstrip()
    morsecode = ''
    for letter in message:
        if letter != ' ':
            morsecode += (MORSE_CODE_DICT[letter] if letter in MORSE_CODE_DICT else letter)
        morsecode += ' '

    if not morsecode.strip():
        raise BadArgument(message="Der Text ist entweder leer oder enthält nur ungültige Zeichen!")
    return morsecode

# Morsecode zu Text
def morse_decrypt(message):
    i = 0
    message += ' '
    text = ''
    char = ''
    for letter in message:
        if letter != ' ':
            i = 0
            char += letter
        else:
            # Neuer Buchstabe
            i += 1
            # Neues Wort
            if i == 2 :
                # Leerzeichen um neues Wort abzutrennen
                text += ' '
            else:
                text += MORSE_CODE_DICT_INVERTED[char] if char in MORSE_CODE_DICT_INVERTED else "*"
                char = ''

    if not text.replace("*","").strip():
        raise BadArgument(message="Der Morsecode ist entweder leer oder enthält nur ungültige Zeichen!")
    return text



from django.urls import path

from . import views

##############

app_name = 'discordbot'
urlpatterns = [
    path('amongus/tracker/post', views.amongus_tracker_post, name="amongus-tracker-post"),
]



# Generated by Django 3.1.2 on 2020-10-31 15:15

import discordbot.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0009_remove_amongusgame_tracker_connected'),
    ]

    operations = [
        migrations.CreateModel(
            name='VierGewinntGame',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('width', models.PositiveSmallIntegerField(default=7, verbose_name='Width')),
                ('height', models.PositiveSmallIntegerField(default=6, verbose_name='Height')),
                ('game', models.JSONField(default=discordbot.models.VIERGEWINNT_DEFAULT_GAME, verbose_name='Game')),
                ('finished', models.BooleanField(default=False, verbose_name='Finished')),
                ('current_player', models.PositiveSmallIntegerField(default=1, verbose_name='Current player')),
                ('player_1_id', models.CharField(max_length=32, verbose_name='Player 1 ID')),
                ('player_2_id', models.CharField(max_length=32, verbose_name='Player 2 ID')),
                ('winner_id', models.CharField(default=None, max_length=32, null=True, verbose_name='Winner ID')),
                ('channel_id', models.CharField(max_length=32, verbose_name='Channel ID')),
                ('message_id', models.CharField(max_length=32, verbose_name='Message ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, verbose_name='Time created')),
                ('time_edited', models.DateTimeField(auto_now=True, verbose_name='Time edited')),
            ],
            options={
                'verbose_name': 'VierGewinnt Game',
                'verbose_name_plural': 'VierGewinnt Games',
            },
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-22 07:03

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0002_auto_20200701_1655'),
    ]

    operations = [
        migrations.CreateModel(
            name='AmongUsGame',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('api_key', models.UUIDField(default=uuid.uuid4, verbose_name='API Key')),
                ('voice_channel_id', models.CharField(max_length=20, verbose_name='Voice channel ID')),
                ('text_channel_id', models.CharField(max_length=20, verbose_name='Text channel ID')),
                ('code', models.CharField(blank=True, default='', max_length=6, verbose_name='Gamecode')),
                ('last_edited', models.DateTimeField(auto_now=True, verbose_name='Last edited')),
                ('tracker_connected', models.DateTimeField(default=False, verbose_name='Tracker connected')),
                ('state_ingame', models.BooleanField(default=False, verbose_name='In game')),
                ('state_meeting', models.BooleanField(default=False, verbose_name='Meeting ongoing')),
                ('p_red_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Red's name")),
                ('p_red_alive', models.BooleanField(default=True, verbose_name="Red's alive")),
                ('p_red_exists', models.BooleanField(default=True, verbose_name='Red exists')),
                ('p_purple_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Purple's name")),
                ('p_purple_alive', models.BooleanField(default=True, verbose_name="Purple's alive")),
                ('p_purple_exists', models.BooleanField(default=True, verbose_name='Purple exists')),
                ('p_orange_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Orange's name")),
                ('p_orange_alive', models.BooleanField(default=True, verbose_name="Orange's alive")),
                ('p_orange_exists', models.BooleanField(default=True, verbose_name='Orange exists')),
                ('p_yellow_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Yellow's name")),
                ('p_yellow_alive', models.BooleanField(default=True, verbose_name="Yellow's alive")),
                ('p_yellow_exists', models.BooleanField(default=True, verbose_name='Yellow exists')),
                ('p_white_name', models.CharField(blank=True, default='', max_length=50, verbose_name="White's name")),
                ('p_white_alive', models.BooleanField(default=True, verbose_name="White's alive")),
                ('p_white_exists', models.BooleanField(default=True, verbose_name='White exists')),
                ('p_black_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Black's name")),
                ('p_black_alive', models.BooleanField(default=True, verbose_name="Black's alive")),
                ('p_black_exists', models.BooleanField(default=True, verbose_name='Black exists')),
                ('p_green_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Green's name")),
                ('p_green_alive', models.BooleanField(default=True, verbose_name="Green's alive")),
                ('p_green_exists', models.BooleanField(default=True, verbose_name='Green exists')),
                ('p_lime_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Lime's name")),
                ('p_lime_alive', models.BooleanField(default=True, verbose_name="Lime's alive")),
                ('p_lime_exists', models.BooleanField(default=True, verbose_name='Lime exists')),
                ('p_blue_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Blue's name")),
                ('p_blue_alive', models.BooleanField(default=True, verbose_name="Blue's alive")),
                ('p_blue_exists', models.BooleanField(default=True, verbose_name='Blue exists')),
                ('p_cyan_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Cyan's name")),
                ('p_cyan_alive', models.BooleanField(default=True, verbose_name="Cyan's alive")),
                ('p_cyan_exists', models.BooleanField(default=True, verbose_name='Cyan exists')),
                ('p_brown_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Brown's name")),
                ('p_brown_alive', models.BooleanField(default=True, verbose_name="Brown's alive")),
                ('p_brown_exists', models.BooleanField(default=True, verbose_name='Brown exists')),
                ('p_pink_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Pink's name")),
                ('p_pink_alive', models.BooleanField(default=True, verbose_name="Pink's alive")),
                ('p_pink_exists', models.BooleanField(default=True, verbose_name='Pink exists')),
                ('creator', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='discordbot.user')),
                ('guild', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='discordbot.server')),
            ],
            options={
                'verbose_name': 'AmongUs Game',
                'verbose_name_plural': 'AmongUs Games',
                'unique_together': {('guild', 'creator')},
            },
        ),
    ]



# pylint: disable=unused-variable

from discord.ext import commands

from discordbot.config import ALLOW_BOTS, ALLOW_WEBHOOKS, DEBUG

def setup(bot):
    @bot.event
    async def on_message(message):
        if message.author != bot.user:
            is_bot = message.author.bot
            is_webhook = bool(message.webhook_id)

            if (not is_bot) or ((ALLOW_BOTS and not is_webhook) or (ALLOW_WEBHOOKS and is_webhook)):
                ctx = await bot.get_context(message)
                a = await bot.invoke(ctx)



# Generated by Django 3.1.2 on 2020-10-22 14:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0004_auto_20201022_0928'),
    ]

    operations = [
        migrations.AddField(
            model_name='amongusgame',
            name='text_message_id',
            field=models.CharField(default=None, max_length=20, null=True, verbose_name='Text message ID'),
        ),
    ]



# pylint: disable=unused-variable

from discord.ext import commands

from discordbot.config import DEBUG

def setup(bot):
    @bot.event
    async def on_command(ctx):
        if DEBUG:
            print(f"[Command] - '{ctx.message.content}' von '{ctx.author.name}#{str(ctx.author.discriminator)}'")
        if ctx.guild is not None:
            try:
                await ctx.message.delete()
            except:
                pass



# Generated by Django 3.1.3 on 2020-11-15 17:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0018_auto_20201114_2032'),
    ]

    operations = [
        migrations.CreateModel(
            name='Playlist',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(default='Server default', max_length=256, verbose_name='Title')),
                ('current_pos', models.PositiveSmallIntegerField(default=0, verbose_name='Current position')),
                ('server', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='playlists', to='discordbot.server')),
            ],
            options={
                'verbose_name': 'Audio queue',
                'verbose_name_plural': 'Audio queues',
            },
        ),
        migrations.CreateModel(
            name='PlaylistPosition',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('position', models.PositiveSmallIntegerField(verbose_name='Position')),
                ('queue', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='discordbot.playlist')),
                ('source', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='discordbot.audiosource')),
            ],
            options={
                'verbose_name': 'Audio queue position',
                'verbose_name_plural': 'Audio queue positions',
            },
        ),
        migrations.AddField(
            model_name='playlist',
            name='sources',
            field=models.ManyToManyField(through='discordbot.PlaylistPosition', to='discordbot.AudioSource'),
        ),
        migrations.AddField(
            model_name='server',
            name='playlist',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='discordbot.playlist'),
        ),
    ]



from discord.ext import commands
from discord import Embed, Member, User, Permissions


class Moderation(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0x5156ff


    @commands.command(
        brief="Leert den Chat",
        description="Leert den Chat",
        aliases=["cc"],
        help="Gib einfach /clearchat ein und der Chat wird bald leer sein",
        usage=""
        )
    @commands.cooldown(1,2.0,commands.BucketType.channel)
    @commands.has_permissions(manage_messages = True)
    @commands.bot_has_permissions(manage_messages = True)
    @commands.guild_only()
    async def clearchat(self,ctx):
        await ctx.sendEmbed(title="Chat wird geleert...", description="Der Chat wird geleert!")
        try:
            await ctx.message.channel.purge()
        except Exception as e:
            print("[Clearchat] - Error:", e)
            pass



    @commands.command(
        brief="Kickt einen Spieler",
        description="Kickt einen Spieler vom Server",
        aliases=[],
        help="Benutze /kick <Member> [Grund] um einen Spieler zu kicken",
        usage="<Member> [Grund]"
        )
    @commands.has_permissions(kick_members = True)
    @commands.bot_has_permissions(kick_members = True)
    @commands.guild_only()
    async def kick(self, ctx, member: Member, *args):
        if ctx.author.roles[-1] > member.roles[-1]:
            Grund = " ".join(args)
            if Grund.rstrip() == "":
                Grund = "Leer"
            await member.kick(reason="Von Moderator "+ctx.author.name+"#"+ctx.author.discriminator+" angefordert: "+Grund)
            await ctx.sendEmbed(title="Benutzer Gekickt", fields=[("Betroffener",member.mention),("Grund",Grund)])
        else:
            raise commands.BadArgument(message="Deine Rolle ist nicht höher als die des Benutzers, den du kicken wolltest!")


    @commands.command(
        brief="Bannt einen Spieler",
        description="Bannt einen Spieler vom Server",
        aliases=[],
        help="Benutze /ban <Member> [Grund] um einen Spieler zu bannen",
        usage="<Member> [Grund]"
        )
    @commands.has_permissions(ban_members = True)
    @commands.bot_has_permissions(ban_members = True)
    @commands.guild_only()
    async def ban(self, ctx, member: Member, *args):
        if ctx.author.roles[-1] > member.roles[-1]:
            Grund = " ".join(args)
            if Grund.rstrip() == "":
                Grund = "Leer"
            await member.ban(reason="Von Moderator "+ctx.author.name+"#"+ctx.author.discriminator+" angefordert: "+Grund)
            await ctx.sendEmbed(title="Benutzer Gebannt", fields=[("Betroffener",member.mention),("Grund",Grund)])
        else:
            raise commands.BadArgument(message="Deine Rolle ist nicht höher als die des Benutzers, den du bannen wolltest!")



    @commands.command(
        brief="Entbannt einen Spieler",
        description="Entbannt einen zuvor gebannten Spieler",
        aliases=["pardon"],
        help="Benutze /unban <Userid> [Grund] um einen Spieler zu entbannen",
        usage="<Userid> [Grund]"
        )
    @commands.has_permissions(ban_members = True)
    @commands.bot_has_permissions(ban_members = True)
    @commands.guild_only()
    async def unban(self, ctx, userid: int, *args):
        Grund = " ".join(args)
        if Grund.rstrip() == "":
            Grund = "Leer"
        user = self.bot.get_user(userid)
        if user is None:
            raise commands.BadArgument(message="Benutzer wurde nicht gefunden!")
        try:
            await ctx.guild.unban(user,reason="Von Moderator "+ctx.author.name+"#"+ctx.author.discriminator+" angefordert: "+Grund)
            await ctx.sendEmbed(title="Benutzer Entbannt", fields=[("Betroffener",user.mention),("Grund",Grund)])
        except:
            raise commands.BadArgument(message="Benutzer wurde nicht gefunden!")


    @commands.command(
        brief="Tötet einen Spieler",
        description="Kickt einen Spieler aus dem aktuellen Sprachkanal",
        aliases=["kickvoice"],
        help="Benutze /kill <Member> [Grund] um einen Spieler zu töten",
        usage="<Member> [Grund]"
        )
    @commands.guild_only()
    async def kill(self, ctx, member: Member, *args):
        Grund = " ".join(args)
        if Grund.rstrip() == "":
            Grund = "Leer"
        VoiceState = member.voice
        if VoiceState:
            if VoiceState.channel.permissions_for(ctx.author).move_members:
                if VoiceState.channel.permissions_for(ctx.guild.get_member(self.bot.user.id)).move_members:
                    if ctx.author.roles[-1] >= member.roles[-1]:
                        await member.edit(voice_channel=None,reason="Von Moderator "+ctx.author.name+"#"+ctx.author.discriminator+" angefordert: "+Grund)
                        await ctx.sendEmbed(title="Benutzer Getötet", fields=[("Betroffener",member.mention),("Grund",Grund)])
                    else:
                        raise commands.BadArgument(message="Deine Rolle ist nicht höher als oder gleich wie die des Benutzers, den du töten wolltest!")
                else:
                    raise commands.BotMissingPermissions([])
            else:
                raise commands.MissingPermissions([])
        else:
            raise commands.BadArgument(message="Der Benutzer befindet sich nicht in einem Sprachkanal.")

    @commands.command(
        brief="Bewegt einen Spieler zu dir",
        description="Bewegt einen Spieler in deinen aktuellen Kanal",
        aliases=["mvhere"],
        help="Benutze /movehere <Member> um ein Mitglied hier hin zu bewegen.",
        usage="<Member>"
        )
    @commands.guild_only()
    async def movehere(self, ctx, member: Member):
        if member.voice:
            if ctx.author.voice:
                if member.voice.channel.permissions_for(ctx.author).move_members:
                    if member.voice.channel.permissions_for(ctx.guild.get_member(self.bot.user.id)).move_members:
                        await member.edit(voice_channel=ctx.author.voice.channel,reason="Von Moderator "+ctx.author.name+"#"+ctx.author.discriminator+" angefordert.")
                        await ctx.sendEmbed(title="Hierhin bewegt", fields=[("Betroffener",member.mention),("Kanal",ctx.author.voice.channel.name)])
                    else:
                        raise commands.BotMissingPermissions([])
                else:
                    raise commands.MissingPermissions([])
            else:
                raise commands.BadArgument(message="Du befindest sich nicht in einem Sprachkanal.")
        else:
            raise commands.BadArgument(message="Der Benutzer befindet sich nicht in einem Sprachkanal.")


def setup(bot):
    bot.add_cog(Moderation(bot))



from discord.ext import commands
from discord import Embed, User, Member, FFmpegPCMAudio, PCMVolumeTransformer, VoiceChannel
from fuzzywuzzy import process
import os

from discordbot.config import RADIOS, FFMPEG_OPTIONS, FILESPATH, MEMESPATH, MUSIC_MODULE
from discordbot.errors import ErrorMessage

#####

class Music(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0xee00ff

    # Events

    @commands.Cog.listener()
    async def on_voice_state_update(self, member, before, after):
        if MUSIC_MODULE:
            # *Grillenzirpen* nach Streamende
            if before.channel and before.self_stream and not after.self_stream:
                filepath = os.path.join(MEMESPATH, "grillenzirpen.wav")
                if os.path.isfile(filepath):
                    voice_client = before.channel.guild.voice_client
                    if voice_client is None:
                        voice_client = await before.channel.connect()
                    elif voice_client.is_playing():
                        voice_client.stop()
                        await voice_client.move_to(before.channel)

                    player = PCMVolumeTransformer(FFmpegPCMAudio(
                        source=filepath, **FFMPEG_OPTIONS))
                    voice_client.play(player, after=lambda e: print(
                        '[Music] - Error: %s' % e) if e else None)

    @commands.Cog.listener()
    async def on_message(self, message):
        if MUSIC_MODULE:
            # *Help I need somebody Help*
            if "i need somebody" in message.content.lower() and message.guild and message.author.voice:
                filepath = os.path.join(MEMESPATH, "help-i-need-somebody.wav")
                if os.path.isfile(filepath):
                    voice_client = message.guild.voice_client
                    if voice_client is None:
                        voice_client = await message.author.voice.channel.connect()
                    elif voice_client.is_playing():
                        voice_client.stop()
                        await voice_client.move_to(message.author.voice.channel)

                    player = PCMVolumeTransformer(FFmpegPCMAudio(
                        source=filepath, **FFMPEG_OPTIONS))
                    voice_client.play(player, after=lambda e: print(
                        '[Music] - Error: %s' % e) if e else None)

    # Utils

    async def ensure_voice(self, ctx):
        if ctx.author.voice and ctx.author.voice.channel.guild == ctx.guild:
            if ctx.voice_client is None:
                await ctx.author.voice.channel.connect()
            #elif ctx.voice_client.is_playing():
                #ctx.voice_client.stop()
            await ctx.voice_client.move_to(ctx.author.voice.channel)
        else:
            raise ErrorMessage("Du bist mit keinem Sprachkanal in diesem Server verbunden!")

    if MUSIC_MODULE:
        @commands.command(
            brief='Liste alle Memes auf',
            description='Liste alle Memes auf',
        )
        async def memes(self, ctx):
            filenames = list(os.listdir(MEMESPATH))
            await ctx.sendEmbed(title="Memes", description="\n".join(["- "+filename.split(".")[0] for filename in filenames]))


        @commands.command(
            brief='Spiele Memes',
            description='Spiele Memes von einer Audiodatei!',
            usage="<Suche>"
        )
        @commands.guild_only()
        async def meme(self, ctx, search:str="windows-xp-error", *args):
            search = " ".join((search,)+args)
            filenames = list(os.listdir(MEMESPATH))

            result = process.extractOne(search, filenames)
            filename = result[0]

            print("[Music] - Suchergebnis:", search, result)

            if result[1] >= 75:
                player = PCMVolumeTransformer(FFmpegPCMAudio(source=os.path.join(MEMESPATH, filename), **FFMPEG_OPTIONS))

                if ctx.voice_client.is_playing():
                    ctx.voice_client.stop()

                ctx.voice_client.play(player, after=lambda e: print('[Music] - Fehler: %s' % e) if e else None)
                await ctx.sendEmbed(title="Memetime!", fields=[("Meme",str(filename).split(".")[0])])
            else:
                raise commands.BadArgument(message="Es wurden keine mit '{}' übereinstimmende Audiodatei gefunden.".format(search))

        @commands.command(
            brief='Spiele Musik',
            description='Spiele Musik von Youtube und anderen Plattformen!',
            aliases=["yt","youtube"],
            usage="<Url/YT Suche>"
        )
        @commands.guild_only()
        async def play(self, ctx, search: str, *args):
            url = " ".join((search,)+args)
            async with ctx.typing():
                player = await ctx.data.musicqueue.createYoutubePlayer(url, loop=self.bot.loop)
                if ctx.voice_client.is_playing():
                    ctx.data.musicqueue.addPlayer(player)
                    await player.send(ctx, status="Song zur Playlist hinzugefügt!")
                else:
                    player.play(ctx)
                    await player.send(ctx)


        @commands.command(
            name='stream',
            brief='Streame einen Stream',
            description='Streame einen Stream von YouTube oder anderen Plattformen',
            aliases=[],
            usage="<Url/YT Suche>"
        )
        @commands.guild_only()
        async def stream(self, ctx, search: str, *args):
            url = " ".join((search,)+args)
            if url in RADIOS:
                url = RADIOS[url]

            async with ctx.typing():
                player = await ctx.data.musicqueue.createYoutubePlayer(url, loop=self.bot.loop, stream=True)

                if ctx.voice_client.is_playing():
                    ctx.voice_client.stop()

                player.play(ctx)
                await player.send(ctx, "Stream wird direkt wiedergegeben!")

        @commands.command(
            brief='Aktueller Titel abrufen',
            description='Sieh, was gerade läuft',
            aliases=["np"],
        )
        @commands.guild_only()
        async def nowplaying(self, ctx):
            await ctx.data.musicqueue.sendNowPlaying(ctx)

        @commands.command(
            brief="Warteschlange abrufen",
            description='Sieh, was als nächstes läuft',
        )
        @commands.guild_only()
        async def queue(self, ctx):
            await ctx.data.musicqueue.sendQueue(ctx)

        @commands.command(
            brief='Pausiere Musik',
            description='Pausiere die aktuelle Musik',
        )
        @commands.guild_only()
        async def pause(self, ctx):
            if ctx.voice_client and ctx.voice_client.is_playing():
                ctx.voice_client.pause()
                await ctx.sendEmbed(title="Musik pausiert")

        @commands.command(
            brief='Führe Musik fort',
            description='Hebe die Pausierung der aktuellen Musik auf.',
        )
        @commands.guild_only()
        async def resume(self, ctx):
            if ctx.voice_client and ctx.voice_client.is_paused():
                ctx.voice_client.resume()
                await ctx.sendEmbed(title="Pausierung aufgehoben")

        @commands.command(
            brief='Überspringe Musik',
            description='Überspringe aktuelle Musik',
            usage="<Url/Suche>"
        )
        @commands.guild_only()
        async def skip(self, ctx):
            async with ctx.typing():
                if ctx.voice_client.is_playing():
                    ctx.voice_client.stop()
                ctx.data.musicqueue.playNext(ctx)
                await ctx.data.musicqueue.sendNowPlaying(ctx)

        @commands.command(
            brief='Ändere die Lautstärke',
            description='Ändere die Lautstärke des Bots',
            aliases=["vol"],
            usage="<1-200>"
        )
        @commands.guild_only()
        async def volume(self, ctx, newvolume: float = None):
            if not ctx.voice_client.source:
                raise ErrorMessage("Der Bot scheint aktuell nichts abzuspielen.")

            oldvolume = ctx.voice_client.source.volume * 100

            if newvolume is None:
                await ctx.sendEmbed(title="Lautstärke", fields=[("Aktuell", str(oldvolume)+"%")])
            else:
                ctx.voice_client.source.volume = newvolume / 100

                await ctx.sendEmbed(title="Lautstärke geändert", fields=[("Zuvor", str(oldvolume)+"%"),("Jetzt",str(newvolume)+"%")])

        @commands.command(
            brief='Stoppe Musik',
            description='Stoppe Musik!',
            aliases=["die","leave","disconnect"],
            help="Benutze /stop um den Bot aus dem Sprachkanal zu entfernen.",
        )
        @commands.guild_only()
        async def stop(self, ctx):
            if ctx.voice_client:
                await ctx.voice_client.disconnect()
                await ctx.sendEmbed(title="Bye bye!")
            else:
                raise ErrorMessage("Der Bot war in gar keinem Sprachkanal!")

        @meme.before_invoke
        @play.before_invoke
        @stream.before_invoke
        @nowplaying.before_invoke
        @skip.before_invoke
        @volume.before_invoke
        async def autojoin(self, ctx):
            await self.ensure_voice(ctx)

    # Generelle Commands

    @commands.command(
        brief='Erhalte Infos über einen Song',
        description='Erhalte Infos über einen Song oder eine Playlist',
        aliases=[],
        hidden=True,
    )
    async def songinfo(self, ctx, query:str):
        async with ctx.typing():
            embdata = await ctx.audio.getInfoEmbedData(query)
            await ctx.sendEmbed(**embdata)

    @commands.command(
        name='usersong',
        brief='Stalke musikhörende Leute',
        description='Erhalte Links zu dem Song, welcher jemand gerade hört',
        aliases=[],
        help="Benutze /usersong <Member> um den Song zu erhalten",
        usage="<Member>"
    )
    @commands.guild_only()
    async def usersong(self, ctx, Member:Member):
        found = False
        for activity in Member.activities:
            if str(activity.type) == "ActivityType.listening":
                try:
                    await ctx.sendEmbed(title="User Song", fields=[("Titel", activity.title),("Künstler", activity.artist),("Link", ("[Spotify](https://open.spotify.com/track/"+activity.track_id+")"))])
                except AttributeError:
                    raise commands.BadArgument(message="Scheinbar hört dieser Benutzer keinen richtigen Song.")
                found = True
        if not found:
            raise commands.BadArgument(message="Dieser Benutzer hört keinen Song!")




def setup(bot):
    bot.add_cog(Music(bot))



# Generated by Django 3.1.2 on 2020-10-22 07:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0003_amongusgame'),
    ]

    operations = [
        migrations.AlterField(
            model_name='amongusgame',
            name='tracker_connected',
            field=models.BooleanField(default=False, verbose_name='Tracker connected'),
        ),
    ]



from discord.ext import commands
from discord import Embed, User, Member, utils, PermissionOverwrite, Role
import typing

CHANNEL_NAMES_DEFAULT = ["Sprachkanal", "Sprachkanal erstellen", "Neuer Sprachkanal", ]
CHANNEL_NAMES_PUBLIC = ["Sprachkanal [offen]", "Sprachkanal erstellen [offen]", "Neuer Sprackanal [offen]", ]
CHANNEL_NAMES_PRIVATE = ["Sprachkanal [privat]", "Sprachkanal erstellen [privat]", "Neuer Sprachkanal [privat]", ]

CATEGORY_NAMES = ["BENUTZERKANÄLE", ]

PERM_VOICE_OWNER = PermissionOverwrite(connect=True, speak=True, view_channel=True, move_members=True, mute_members=True, deafen_members=True, stream=True, priority_speaker=True, use_voice_activation=True, create_instant_invite=True)
PERM_VOICE_PRIVATE = PermissionOverwrite(connect=False, speak=True, view_channel=True)
PERM_VOICE_PUBLIC = PermissionOverwrite(connect=True, speak=True, view_channel=True)

PERM_TEXT_OWNER = PermissionOverwrite(read_messages=True, read_message_history=True, send_messages=True, manage_messages=True, send_tts_messages=True, manage_webhooks=True, add_reactions=True, embed_links=True, attach_files=True, create_instant_invite=True, external_emojis=True)
PERM_TEXT_PRIVATE = PermissionOverwrite(read_messages=False)

async def getUserChannelCategory(guild):
    category = utils.get(guild.categories, name="Benutzerkanäle")
    if not category:
        categoryoverwrites = { guild.default_role: PermissionOverwrite(read_messages=False, send_messages=False, connect=False, speak=False, move_members=False, use_voice_activation=True) }
        textchanneloverwrites = { guild.default_role: PermissionOverwrite(read_messages=True, send_messages=True) }
        voicechanneloverwrites = { guild.default_role: PermissionOverwrite(view_channel=True, connect=True, speak=False) }
        category = await guild.create_category_channel(name="Benutzerkanäle", overwrites=categoryoverwrites, reason="Bereite Benutzerkanäle vor...")
        await category.create_text_channel(name="benutzerkanäle", overwrites=textchanneloverwrites, reason="Bereite Benutzerkanäle vor...", topic="Hilfe: /help channels")
        await category.create_voice_channel(name=CHANNEL_NAMES_PUBLIC[0], overwrites=voicechanneloverwrites, reason="Bereite Benutzerkanäle vor...")
        await category.create_voice_channel(name=CHANNEL_NAMES_PRIVATE[0], overwrites=voicechanneloverwrites, reason="Bereite Benutzerkanäle vor...")
    return category

class Channels(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0xee00ff
        
    @commands.Cog.listener()
    async def on_voice_state_update(self, member, before, after):
        category = await getUserChannelCategory(member.guild)

        # Delete channel if empty
        if before.channel and before.channel.category and before.channel.category.name.upper() in CATEGORY_NAMES and "#" in before.channel.name and before.channel.members == []:
            await before.channel.delete(reason="Kanal war leer")

        # Create new channel
        if after.channel and after.channel.category and after.channel.category.name.upper() in CATEGORY_NAMES and after.channel.name in CHANNEL_NAMES_DEFAULT+CHANNEL_NAMES_PRIVATE+CHANNEL_NAMES_PUBLIC:
            channel = utils.get(member.guild.voice_channels, name=(member.name+"#"+member.discriminator))
            if channel:
                await member.edit(voice_channel=channel,reason="Benutzer wollte einen Kanal erstellen, besitzte aber bereits einen Kanal")
            else:
                if after.channel.name in CHANNEL_NAMES_PRIVATE:
                    overwrites = { member.guild.default_role: PERM_VOICE_PRIVATE, member: PERM_VOICE_OWNER}
                else:
                    overwrites = { member.guild.default_role: PERM_VOICE_PUBLIC, member: PERM_TEXT_OWNER }
                newchannel = await category.create_voice_channel(name=(member.name+"#"+member.discriminator),overwrites=overwrites,reason="Benutzer hat den Sprachkanal erstellt")
                await member.edit(voice_channel=newchannel,reason="Benutzer hat den Sprachkanal erstellt")

    # Textchannels

    @commands.group(
        brief="Hauptcommand für alle Textchannel Befehle",
        description='Erstelle deinen eigenen Textkanal und lade deine Freunde oder den ganzen Server dazu ein',
        aliases=['tc'],
        usage="<Unterbefehl> [Argument(e)]"
    )
    @commands.bot_has_permissions(manage_channels=True)
    @commands.guild_only()
    async def textchannel(self, ctx):
        if ctx.invoked_subcommand is None:
            await ctx.send_help()


    @textchannel.command(
        name="create",
        brief='Erstelle deinen Textkanal',
        aliases=[],
    )
    async def textchannel_create(self, ctx):
        category = await getUserChannelCategory(ctx.guild)
        channel = utils.get(ctx.guild.text_channels, name=(ctx.author.name+"-"+ctx.author.discriminator), category=category)
        if channel:
            raise commands.BadArgument(message="Du hast bereits einen Textkanal! <#"+str(channel.id)+">")
        else:
            overwrites = { ctx.guild.default_role: PERM_TEXT_PRIVATE, ctx.author: PERM_TEXT_OWNER }
            newchannel = await category.create_text_channel(name=(ctx.author.name+"-"+ctx.author.discriminator), overwrites=overwrites, reason="Benutzer hat den Textkanal erstellt")
            await ctx.sendEmbed(title="Textkanal erstellt", fields=[("Kanal", newchannel.mention)])


    @textchannel.command(
        name="delete",
        brief='Lösche deinen Textkanal',
        aliases=['del'],
    )
    async def textchannel_delete(self, ctx):
        category = await getUserChannelCategory(ctx.guild)
        channel = utils.get(ctx.guild.text_channels, name=(ctx.author.name+"-"+ctx.author.discriminator), category=category)
        if channel:
            await channel.delete(reason="Benutzer hat den Textkanal gelöscht")
            if not ctx.channel == channel:
                await ctx.sendEmbed(title="Textkanal gelöscht", fields=[("Server", ctx.guild.name)])
        else:
            raise commands.BadArgument(message="Du hattest gar keinen Textkanal!")
        return

    @textchannel.command(
        name="invite",
        brief='Lade jemanden in deinen Textkanal ein',
        description='Lade ein Mitglied oder eine Rolle in deinen Textkanal ein',
        aliases=["add", "+"],
        usage="<Mitglied/Rolle>",
    )
    async def textchannel_invite(self, ctx, wer: typing.Union[Member, Role]):
        category = await getUserChannelCategory(ctx.guild)
        channel = utils.get(ctx.guild.text_channels, name=(
            ctx.author.name+"-"+ctx.author.discriminator), category=category)
        if not channel:
            raise commands.BadArgument(
                message="Du hast noch keinen Textkanal!")
        else:
            await channel.set_permissions(wer, reason="Benuter hat Benutzer/Rolle eingeladen", read_messages=True, send_messages=True)
            if isinstance(wer, Member):
                await ctx.sendEmbed(title="Benutzer zu Textkanal eingeladen", fields=[("Benutzer", wer.mention)])
                #EMBED = ctx.getEmbed(title="Benutzer zu Textkanal eingeladen", fields=[("Server", ctx.guild.name), ("Benutzer", wer.mention)])
                #await ctx.author.send(embed=EMBED)
                await channel.send(wer.mention+" wurde zu diesem Textkanal hinzugefügt.")
            elif isinstance(wer, Role):
                await ctx.sendEmbed(title="Rolle zu Textkanal eingeladen", fields=[("Rolle", wer.name)])
                #EMBED = ctx.getEmbed(title="Rolle zu Textkanal eingeladen", fields=[("Server", ctx.guild.name), ("Rolle", wer.name)])
                #await ctx.author.send(embed=EMBED)
                await channel.send("Alle mit der Rolle "+wer.mention+" wurden zu diesem Textkanal hinzugefügt.")
        return

    @textchannel.command(
        name="open",
        brief='Öffne deinen Textkanal für alle Mitglieder dieses Servers',
        aliases=["publish", "pub"],
    )
    async def textchannel_open(self, ctx):
        category = await getUserChannelCategory(ctx.guild)
        channel = utils.get(ctx.guild.text_channels, name=(
            ctx.author.name+"-"+ctx.author.discriminator), category=category)
        if not channel:
            raise commands.BadArgument(
                message="Du hast noch keinen Textkanal!")
        else:
            await channel.set_permissions(ctx.guild.default_role, reason="Benuter hat den Kanal für alle Geöffnet", read_messages=True, send_messages=True)
            await ctx.sendEmbed(title="Kanal geöffnet", description="Der Kanal ist nun für alle auf diesem Server geöffnet!")

    @textchannel.command(
        name="close",
        brief='Schliesse deinen Textkanal für alle Mitglieder dieses Servers',
        aliases=["unpublish", "unpub"],
    )
    async def textchannel_close(self, ctx):
        category = await getUserChannelCategory(ctx.guild)
        channel = utils.get(ctx.guild.text_channels, name=(
            ctx.author.name+"-"+ctx.author.discriminator), category=category)
        if not channel:
            raise commands.BadArgument(
                message="Du hast noch keinen Textkanal!")
        else:
            await channel.set_permissions(ctx.guild.default_role, reason="Benuter hat den Kanal nicht mehr für alle geöffnet!", read_messages=False, send_messages=False)
            await ctx.sendEmbed(title="Kanal geschlossen", description="Der Kanal ist nun nicht mehr für alle auf diesem Server geöffnet!")

    # Voicechannels

    @commands.group(
        brief="Hauptcommand für alle Voicechannel Befehle",
        description='Erstelle deinen eigenen Sprachkanal und lade deine Freunde oder den ganzen Server dazu ein',
        aliases=['vc'],
        usage="<Unterbefehl> [Argument(e)]"
    )
    @commands.bot_has_permissions(manage_channels=True)
    @commands.guild_only()
    async def voicechannel(self, ctx):
        if ctx.invoked_subcommand is None:
            await ctx.send_help()

    @voicechannel.command(
        name="create",
        brief='Erstelle deinen Sprachkanal',
        aliases=[],
    )
    async def voicechannel_create(self, ctx):
        category = await getUserChannelCategory(ctx.guild)
        channel = utils.get(ctx.guild.voice_channels, name=(ctx.author.name+"#"+ctx.author.discriminator), category=category)
        if channel:
            if ctx.author.voice:
                await ctx.author.edit(voice_channel=channel,reason="Benutzer hat den Kanal erstellt")
            raise commands.BadArgument(message="Du hast bereits einen Sprachkanal!")
        else:
            overwrites = {ctx.guild.default_role: PERM_VOICE_PRIVATE, ctx.author: PERM_VOICE_OWNER}
            newchannel = await category.create_voice_channel(name=(ctx.author.name+"#"+ctx.author.discriminator), overwrites=overwrites, reason="Benutzer hat den Sprachkanal erstellt")
            await ctx.sendEmbed(
                title="Sprachkanal erstellt!",
                footertext="Kanal von USER"
            )
            if ctx.author.voice:
                await ctx.author.edit(voice_channel=newchannel,reason="Benutzer hat den Sprachkanal erstellt")
        return
    
    @voicechannel.command(
        name="delete",
        brief='Lösche deinen Sprachkanal',
        aliases=['del'],
    )
    async def voicechannel_delete(self, ctx):
        category = await getUserChannelCategory(ctx.guild)
        channel = utils.get(ctx.guild.voice_channels, name=(ctx.author.name+"#"+ctx.author.discriminator), category=category)
        if channel:
            await channel.delete(reason="Vom Benutzer gelöscht")
            await ctx.sendEmbed(
                title="Sprachkanal gelöscht!",
                footertext="Kanal von USER",
                fields=[
                    ("Server", ctx.guild.name)
                ]
            )
        else:
            raise commands.BadArgument(message="Du hattest gar keinen Sprachkanal!")
        return


    @voicechannel.command(
        name="invite",
        brief='Lade jemanden in deinen Sprachkanal ein',
        description='Lade ein Mitglied oder eine Rolle in deinen Sprachkanal ein',
        aliases=["add", "+"],
        usage="<Mitglied/Rolle>",
    )
    async def voicechannel_invite(self, ctx, wer: typing.Union[Member,Role]):
        category = await getUserChannelCategory(ctx.guild)
        channel = utils.get(ctx.guild.voice_channels, name=(ctx.author.name+"#"+ctx.author.discriminator), category=category)
        if not channel:
            raise commands.BadArgument(message="Du hast noch keinen Sprachkanal!")
        else:
            await channel.set_permissions(wer,reason="Benuter hat Benutzer/Rolle eingeladen", read_messages=True, connect=True, speak=True)
            if isinstance(wer, Member):
                await ctx.sendEmbed(title="Benutzer zu Sprachkanal eingeladen", fields=[("Benutzer", wer.mention)])
                #EMBED = ctx.getEmbed(title="Benutzer zu Sprachkanal eingeladen", fields=[("Server", ctx.guild.name),("Benutzer", wer.mention)])
                #await ctx.author.send(embed=EMBED)
                if not wer.bot:
                    EMBED2 = ctx.getEmbed(title="Du wurdest zu einem Sprachkanal eingeladen", fields=[("Server", ctx.guild.name),("Von", ctx.author.mention)])
                    await wer.send(embed=EMBED2)
            elif isinstance(wer, Role):
                await ctx.sendEmbed(title="Rolle zu Sprachkanal eingeladen", fields=[("Rolle", wer.name)])
                #EMBED = ctx.getEmbed(title="Rolle zu Sprachkanal eingeladen", fields=[("Server", ctx.guild.name),("Rolle", wer.name)])
                #await ctx.author.send(embed=EMBED)
                await ctx.send("Alle mit der Rolle "+wer.mention+" wurden von "+ctx.author.mention+" zu seinem/ihrem Sprachkanal eingeladen.")
        return

    @voicechannel.command(
        name="open",
        brief='Öffne deinen Sprachkanal für alle Mitglieder dieses Servers',
        aliases=["publish", "pub"],
    )
    async def voicechannel_open(self, ctx):
        category = await getUserChannelCategory(ctx.guild)
        channel = utils.get(ctx.guild.voice_channels, name=(
            ctx.author.name+"#"+ctx.author.discriminator), category=category)
        if not channel:
            raise commands.BadArgument(
                message="Du hast noch keinen Sprachkanal!")
        else:
            await channel.set_permissions(ctx.guild.default_role, reason="Benuter hat den Kanal für alle Geöffnet", read_messages=True, connect=True, speak=True)
            await ctx.sendEmbed(title="Kanal geöffnet", description="Der Sprachkanal ist nun für alle auf diesem Server geöffnet!")

    @voicechannel.command(
        name="close",
        brief='Schliesse deinen Sprachkanal für alle Mitglieder dieses Servers',
        aliases=["unpublish", "unpub"],
    )
    async def voicechannel_close(self, ctx):
        category = await getUserChannelCategory(ctx.guild)
        channel = utils.get(ctx.guild.voice_channels, name=(
            ctx.author.name+"#"+ctx.author.discriminator), category=category)
        if not channel:
            raise commands.BadArgument(
                message="Du hast noch keinen Sprachkanal!")
        else:
            await channel.set_permissions(ctx.guild.default_role, reason="Benuter hat den Kanal nicht mehr für alle geöffnet!", read_messages=True, connect=False, speak=True)
            await ctx.sendEmbed(title="Kanal geschlossen", description="Der Sprachkanal ist nun für alle auf diesem Server geöffnet!")



def setup(bot):
    bot.add_cog(Channels(bot))



# pylint: disable=unused-variable

from discord.ext import commands

from discordbot.config import DEBUG, EXTENSIONFOLDER, EXTENSIONS
from discordbot.botmodules.serverdata import DjangoConnection

def setup(bot):
    @bot.event
    async def on_ready():
        print(f"[Bot] - Logged in as '{bot.user.name}' - '{bot.user.id}'")
        for extension in EXTENSIONS:
            try:
                bot.load_extension(EXTENSIONFOLDER+"."+extension)
            except commands.errors.ExtensionAlreadyLoaded:
                pass
        print(f"[Bot] - Loaded extensions!")

    @bot.event
    async def on_connect():
        print(f"[Bot] - Connected!")

    @bot.event
    async def on_disconnect():
        print(f"[Bot] - Disconnected!")

    @bot.event
    async def on_guild_join(guild):
        print(f"[Bot] - Joined new guild: {guild.name} ({guild.id})")
        await DjangoConnection.fetch_server(guild)


from discord.ext import commands
from discord import Game, Streaming, Activity, ActivityType, Status, Member, User, TextChannel, VoiceChannel, Role, Invite, Game, Emoji, PartialEmoji, Colour

from discordbot.config import EXTENSIONS, EXTENSIONFOLDER

import typing

class Owneronly(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0x000000


    @commands.command(
        brief="Lade die Bot-Extensionen neu",
    )
    @commands.is_owner()
    async def reload(self, ctx, extension:str=None):
        msg = await ctx.sendEmbed(title="Reload", fields=[("Status", "Reloading")])
        EMBED = ctx.getEmbed(title="Reload", fields=[])
        if extension in EXTENSIONS:
            print("[Bot] - Reloading '"+extension+"' extension...")
            try:
                self.bot.unload_extension(EXTENSIONFOLDER+"."+extension)
            except commands.errors.ExtensionNotLoaded:
                pass
            try:
                self.bot.load_extension(EXTENSIONFOLDER+"."+extension)
            except commands.errors.ExtensionAlreadyLoaded:
                pass
            EMBED.add_field(name="Status",value="Reloaded category "+extension.upper()+"!")
        else:
            print("[Bot] - Reloading all extensions...")
            for extension in EXTENSIONS:
                try:
                    self.bot.unload_extension(EXTENSIONFOLDER+"."+extension)
                except commands.errors.ExtensionNotLoaded:
                    pass
                try:
                    self.bot.load_extension(EXTENSIONFOLDER+"."+extension)
                except commands.errors.ExtensionAlreadyLoaded:
                    pass
            EMBED.add_field(name="Status",value="Reloaded all categories!")
        await msg.edit(embed=EMBED)
        print("[Bot] - Reload completed!")


    @commands.command(
        brief="Stoppe den Bot",
    )
    @commands.is_owner()
    async def stopbot(self, ctx):
        await ctx.sendEmbed(title="Stop", fields=[("Status", "Gestoppt")])
        await self.bot.logout()


    @commands.command(
        brief="Ändere den Status des Bots",
        usage="<Status> [Aktivität [Argumente(e)]]",
    )
    @commands.is_owner()
    async def status(self, ctx, STATUS:str="", ACTIVITY:str="", arg1:str="", *args):
        arg2 = " ".join(args)
        status = None
        activity = None
        if STATUS.lower() in ["on","online","green"]:
            status = Status.online
        elif STATUS.lower() in ["off","offline","invisible","grey"]:
            status = Status.invisible
        elif STATUS.lower() in ["dnd","donotdisturb","do_not_disturb","bittenichtstören","red"]:
            status = Status.dnd
        elif STATUS.lower() in ["idle","abwesend","orange","yellow"]:
            status = Status.idle

        if ACTIVITY.lower() in ["playing","spielt","game","play"]:
            activity=Game(name=arg1+" "+arg2)
        elif ACTIVITY.lower() in ["streaming","streamt","stream","live","twitch"]:
            activity=Streaming(url="https://twitch.tv/"+arg1, name=arg2)
        elif ACTIVITY.lower() in ["listening","listen","hört","hören","song"]:
            activity=Activity(type=ActivityType.listening, name=arg1+" "+arg2)
        elif ACTIVITY.lower() in ["watching","watch","schaut","video"]:
            activity=Activity(type=ActivityType.watching, name=arg1+" "+arg2)

        if status is not None or activity is not None:
            await ctx.bot.change_presence(status=status, activity=activity)
        else:
            await ctx.sendEmbed(title="Status ändern", inline=False,
            description="""
            **Syntax:**
            /status <STATUS> [AKTIVITÄT [ARGUMENT(E)]]

            **Mögliche Status:**
            on/online (Online)
            idle (Abwesend)
            dnd (Bitte nicht stören)
            off/offline (Unsichtbar)

            **Mögliche Aktivitäten:**
            spielt <SPIEL>
            streamt <TWITCH-NAME> <SPIEL>
            hört <SONG>
            schaut <VIDEO>
            """
            )


    @commands.command(
        brief="Führe einen Command als jemanden anderes aus",
        usage="<Member> <Command> [Argumente]",
        aliases=["runas"],
        help="Falls Unterbefehle verwendet werden, benutze bitte befehl_unterbefehl als Command",
    )
    @commands.is_owner()
    async def sudo(self, ctx, member: typing.Union[Member, User], command: str, *args):
        await ctx.invoke_as(member, command, *args)

    @commands.command(
        brief="Erhalte diesen Chat",
        usage="[Messages = 100]",
        aliases=["archiv", "log"],
        hidden=True,
    )
    @commands.is_owner()
    @commands.guild_only()
    async def archive(self, ctx, messages: int = 100):
        msgs = []
        text = ""
        async for msg in ctx.channel.history(limit=messages, oldest_first=True):
            msgs.append(msg)
            time = msg.created_at.strftime("%Y/%m/%d - %H:%M:%S")
            text += f"[[{time}]]({msg.jump_url}) {msg.author.name}#{msg.author.discriminator}: {msg.content}\n"
        await ctx.sendEmbed(
            title="Kanalarchiv",
            description=f"Kanal: {ctx.channel.mention}\nNachrichten: {len(msgs)}\n\n"+text,
            receiver=ctx.author,
        )

def setup(bot):
    bot.add_cog(Owneronly(bot))



# pylint: disable=no-member

from asgiref.sync import sync_to_async

from discord.ext import commands, tasks
from discord import Embed, utils, PermissionOverwrite, Color, NotFound, User, Member

from django.utils import timezone

from datetime import timedelta

from discordbot.models import AmongUsGame, AMONGUS_PLAYER_COLORS, AMONGUS_EMOJI_COLORS, VierGewinntGame, VIERGEWINNT_NUMBER_EMOJIS
from discordbot.botmodules.serverdata import DjangoConnection
from discordbot.botmodules import apis

import requests
import os
import asyncio
import typing

#####

OVERRIDE_MUTED = PermissionOverwrite(
    read_messages=True, connect=True, speak=False, use_voice_activation=True)
OVERRIDE_TALK = PermissionOverwrite(
    read_messages=True, connect=True, speak=True, use_voice_activation=True)


ALIVE = '❤'
DEAD = '💀'
UNKNOWN = '❓'

YES = '✅'
NO = '❌'

DELETE = '❌'

RUNNING = '⏰'

AMONGUS_TRACKER_INSTALL = """Konfiguriere den Tracker mit folgenden Daten:
                
Der Tracker kann [hier](https://raw.githubusercontent.com/rafaelurben/django-discordbot/master/discordbot/files/amongus/tracker.py) heruntergeladen werden. Er funktioniert vorerst nur für Windows uns ist (noch) buggy!
Bisher ist er auch auf die Bildschirmgrösse 1920x1080p beschränkt. Getestet wurde mit LDPlayer im Vollbildmodus.
Benötigt werden Python, Pip und alle [requirements](https://raw.githubusercontent.com/rafaelurben/django-discordbot/master/discordbot/files/amongus/requirements.txt)."""

amongus_last_update = timezone.now()-timedelta(days=1)

#####

async def getAmongUsCategory(guild):
    category = utils.get(guild.categories, name="AmongUs")
    if not category:
        categoryoverwrites = {guild.default_role: PermissionOverwrite(read_messages=True, send_messages=False, connect=True, speak=True, use_voice_activation=True)}
        category = await guild.create_category_channel(name="AmongUs", overwrites=categoryoverwrites, reason="Bereite AmongUs-Kanäle vor...")
    return category


class Games(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0x1f871e

        self.amongus_backgroundtasks.start() 

    def cog_unload(self):
        print("[AmongUs Background Tasks] - Stopped!")
        self.amongus_backgroundtasks.cancel()

    @commands.Cog.listener()
    async def on_raw_reaction_add(self, payload):
        if not payload.user_id == self.bot.user.id:
            emoji = payload.emoji.name
            channel = await self.bot.fetch_channel(payload.channel_id)
            message = await channel.fetch_message(payload.message_id)

            ### AmongUs

            if (emoji in AMONGUS_EMOJI_COLORS or emoji == DELETE) and await DjangoConnection._hasAmongUsGame(text_message_id=payload.message_id):
                try:
                    await message.remove_reaction(emoji, payload.member)
                except:
                    pass

                game = await DjangoConnection._getAmongUsGame(text_message_id=payload.message_id)

                if emoji == DELETE:
                    await game.remove_user(userid=payload.user_id)
                else:
                    c = AMONGUS_EMOJI_COLORS[payload.emoji.name]
                    await game.set_user(userid=payload.user_id, color=c)

            ### VierGewinnt

            if (emoji in VIERGEWINNT_NUMBER_EMOJIS) and await DjangoConnection._has(VierGewinntGame, message_id=payload.message_id):
                try:
                    await message.remove_reaction(emoji, payload.member)
                except:
                    pass

                game = await DjangoConnection._get(VierGewinntGame, message_id=payload.message_id)

                if not game.finished:
                    n = VIERGEWINNT_NUMBER_EMOJIS.index(emoji)

                    if game.process(n, payload.user_id):
                        await DjangoConnection._save(game)

                        embed = self.bot.getEmbed(
                            title=f"Vier Gewinnt (#{game.pk})",
                            color=0x0078D7,
                            description=game.get_description()
                        )

                        await message.edit(embed=embed)

                    if game.process_bot():
                        await DjangoConnection._save(game)

                        embed = self.bot.getEmbed(
                            title=f"Vier Gewinnt (#{game.pk})",
                            color=0x0078D7,
                            description=game.get_description()
                        )

                        await message.edit(embed=embed)

                    if game.finished:
                        for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
                            await message.remove_reaction(emoji, self.bot.user)


    # Fortnite

    @commands.group(
        brief="Hauptcommand für alle Fortnite Befehle",
        description='Erhalte aktuelle Fortnite-Infos',
        aliases=['fn'],
        usage="<Unterbefehl> [Argument(e)]"
    )
    async def fortnite(self, ctx):  
        if ctx.invoked_subcommand is None:
            await ctx.send_help()

    @fortnite.command(
        name="store",
        brief="Erhalte den aktuellen Fortnite Store",
        aliases=['shop']
    )
    async def fortnite_store(self, ctx):
        JSON = apis.Fortnite.getStore()
        await ctx.sendEmbed(
            title="Fortnite Item Shop",
            authorurl="http://fortnitetracker.com/",
            authorname="Powered by Fortnitetracker"
        )
        for i in range(len(JSON)):
            await ctx.sendEmbed(
                title=str(JSON[i]["name"]),
                description=("Rarity: %s \n vBucks: %s" % (JSON[i]["rarity"],JSON[i]["vBucks"])),
                thumbnailurl=str(JSON[i]["imageUrl"]),
                footertext="",
                footerurl="",
            )

    @fortnite.command(
        name="challenges",
        brief="Erhalte die aktuellen Challenges",
        aliases=[]
    )
    async def fortnite_challenges(self, ctx):
        JSON = apis.Fortnite.getChallenges()
        await ctx.sendEmbed(
            title="Fortnite Challenges",
            fields=[((JSON[i]["metadata"][1]["value"]+" ("+JSON[i]["metadata"][3]["value"]+")"),(JSON[i]["metadata"][5]["value"]+" Battlepassstars")) for i in range(len(JSON))],
            thumbnailurl=str(JSON[0]["metadata"][4]["value"]),
            authorurl="http://fortnitetracker.com/",
            authorname="Powered by Fortnitetracker",
            inline=False
        )

    @fortnite.command(
        name="stats",
        brief="Erhalte die Stats eines Spielers",
        aliases=[],
        help="Mögliche Plattformen: [kbm, gamepad, mouse]",
        usage="<Plattform> <Spielername>",
    )
    async def fortnite_stats(self, ctx, platform:str, playername:str):
        JSON = apis.Fortnite.getStats(platform, playername)
        try:
            await ctx.sendEmbed(
                title="Fortnite Stats von "+JSON["epicUserHandle"]+" auf "+JSON["platformNameLong"],
                description=("Account Id: "+JSON["accountId"]),
                fields=[(JSON["lifeTimeStats"][i]["key"], JSON["lifeTimeStats"][i]["value"]) for i in range(len(JSON["lifeTimeStats"]))],
                authorurl="http://fortnitetracker.com/",
                authorname="Powered by Fortnitetracker"
            )
        except KeyError:
            raise commands.BadArgument(message="Spieler wurde auf der angegebenen Platform nicht gefunden!")


    # Minecraft

    @commands.group(
        brief="Hauptcommand für alle Minecraft Befehle",
        description='Erhalte Infos über Minecraft-Spieler',
        aliases=['mc'],
        usage="<Unterbefehl> <Argument>"
    )
    async def minecraft(self, ctx):
        if ctx.invoked_subcommand is None:
            await ctx.send_help()

    @minecraft.command(
        name="uuid",
        brief="Erhalte die UUID eines Spielers",
        aliases=['id'],
        usage="<Spielername>",
    )
    async def minecraft_uuid(self, ctx, name):
        api = apis.Minecraft

        JSON = api.getProfile(name)
        EMBED = ctx.getEmbed(title="Minecraft UUID", fields=[("UUID", JSON["id"], False),("Aktueller Name", JSON["name"], False)])
        if "legacy" in JSON:
            EMBED.add_field(name="Account",value="Alter Account")
        if "demo" in JSON:
            EMBED.add_field(name="Account",value="Demo Account")
        await ctx.send(embed=EMBED)

    @minecraft.command(
        name="names",
        brief="Erhalte alte Namen eines Spielers",
        aliases=['namen', 'name'],
        usage="<UUID>",
    )
    async def minecraft_names(self, ctx, uuid):
        api = apis.Minecraft

        if len(uuid) != 32:
            raise commands.BadArgument("Eine UUID ist genau 32 Zeichen lang!")

        JSON = api.getProfiles(uuid)
        EMBED = ctx.getEmbed(title="Minecraft Namen", description="Sortierung: Von neu bis alt.")
        for i in JSON[::-1]:
            if "changedToAt" in i:
                EMBED.add_field(name="Name seit "+str(i["changedToAt"]),value=i["name"], inline=False)
            else:
                EMBED.add_field(name="Ursprünglicher Name",value=i["name"], inline=False)
        await ctx.send(embed=EMBED)

    @minecraft.command(
        name="skin",
        brief="Erhalte den Skin eines Spielers",
        aliases=[],
        usage="<UUID>",
    )
    async def minecraft_skin(self, ctx, uuid):
        api = apis.Minecraft

        if len(uuid) != 32:
            raise commands.BadArgument("Eine UUID ist genau 32 Zeichen lang!")

        JSON = api.getSkin(uuid)
        EMBED = ctx.getEmbed(title="Minecraft Skin", fields=[("Aktueller Name", JSON["name"]), ("UUID", JSON["id"])])
        if JSON["skin"] is not None:
            EMBED.set_thumbnail(url=JSON["skin"])
        else:
            EMBED.add_field(name="Skin",value="Wurde nicht gefunden. (Steve/Alex)", inline=False)
        await ctx.send(embed=EMBED)

    @minecraft.command(
        name="player",
        brief="Erhalte alle Infos über einen Spieler",
        description="Dies ist eine Zusammenfassung von UUID, Namen und Skin",
        aliases=['spieler'],
        usage="<Spielername>",
    )
    async def minecraft_player(self, ctx, name):
        api = apis.Minecraft
        
        JSON = api.getProfile(name)
        UUID = JSON["id"]
        EMBED = ctx.getEmbed(title="Minecraft Spieler", fields=[("UUID", UUID)], inline=False)
        if "legacy" in JSON:
            EMBED.add_field(name="Account",value="Alter Account")
        if "demo" in JSON:
            EMBED.add_field(name="Account",value="Demo Account")
        JSON2 = api.getProfiles(UUID)
        for i in JSON2[::-1]:
            if "changedToAt" in i:
                EMBED.add_field(name="Name seit "+str(i["changedToAt"]),value=i["name"], inline=False)
            else:
                EMBED.add_field(name="Ursprünglicher Name",value=i["name"], inline=False)
        JSON3 = api.getSkin(UUID)
        if JSON3["skin"] is not None:
            EMBED.set_thumbnail(url=JSON3["skin"])
        else:
            EMBED.add_field(name="Skin",value="Wurde nicht gefunden. (Steve/Alex)", inline=False)
        await ctx.send(embed=EMBED)



    # AmongUs
    ## AmongUs Tasks

    @tasks.loop(seconds=0.5)
    async def amongus_backgroundtasks(self):
        def log(*args):
            print('[AmongUs Background Tasks] -', *args)

        global amongus_last_update
        games = await DjangoConnection._list(AmongUsGame, last_edited__gt=amongus_last_update)
        amongus_last_update = timezone.now()

        for game in games:
            try:
                log(f'Updating AmongUs #{ game.pk }...')

                # Get channels / Create channels

                textchannel = self.bot.get_channel(int(game.text_channel_id))
                voicechannel = self.bot.get_channel(int(game.voice_channel_id))

                if voicechannel is None:
                    log(f"- Deleted AmongUs #{ game.pk } because voicechannel was not found!")
                    await DjangoConnection._delete(game)
                    if textchannel is not None:
                        await textchannel.delete()
                    continue
                elif textchannel is None:
                    log(f"- Created new textchannel for AmongUs #{ game.pk }!")
                    category = await getAmongUsCategory(voicechannel.guild)
                    textchannel = await category.create_text_channel(name=f"amongus-{ game.pk }", reason="Textkanal war nicht mehr vorhanden!", topic=f"AmongUs Spiel - ID: { game.pk }")
                
                    game.text_channel_id = str(textchannel.id)
                    await DjangoConnection._save(game)

                try:
                    msg = await textchannel.fetch_message(int(game.text_message_id))
                except:
                    log(f"- Created new message for AmongUs #{ game.pk }!")
                    embed = self.bot.getEmbed(
                        title=f"AmongUs Game { game.pk }",
                        color=0xFEDE29,
                        footerurl=None
                    )
                    msg = await textchannel.send(embed=embed)

                    game.text_message_id = str(msg.id)
                    await DjangoConnection._save(game)

                    for emoji in AMONGUS_EMOJI_COLORS:
                        await msg.add_reaction(emoji)
                    await msg.add_reaction(DELETE)

                guild = textchannel.guild

                # Mute / Unmute

                vss = voicechannel.voice_states

                if game.state_ingame:
                    if game.state_meeting:
                        # Unmute alive players
                        for c in AMONGUS_PLAYER_COLORS:
                            if getattr(game, f"p_{c}_exists") and getattr(game, f"p_{c}_alive"):
                                m_id = getattr(game, f"p_{c}_userid")
                                if m_id and (int(m_id) in vss):
                                    vs = vss[int(m_id)]
                                    if vs.mute:
                                        m = vs.channel.guild.get_member(int(m_id))
                                        if m is None:
                                            m = await vs.channel.guild.fetch_member(int(m_id))
                                        await m.edit(mute=False)
                    else:
                        # Mute all players
                        for m_id in vss:
                            vs = vss[m_id]
                            if not vs.mute:
                                m = vs.channel.guild.get_member(m_id)
                                if m is None:
                                    m = await vs.channel.guild.fetch_member(m_id)
                                if not m.bot:
                                    await m.edit(mute=True)

                    # Don't allow new players to speak
                    overwrites = {
                        guild.default_role: OVERRIDE_MUTED
                    }
                    await voicechannel.edit(overwrites=overwrites)
                else:
                    # Unmute all players
                    for m_id in vss:
                        vs = vss[m_id]
                        if vs.mute:
                            m = vs.channel.guild.get_member(m_id)
                            if m is None:
                                m = await vs.channel.guild.fetch_member(m_id)
                            await m.edit(mute=False)

                    # Allow new players to speak
                    await voicechannel.edit(sync_permissions=True)

                # Message

                d = game.get_data()

                id = d["id"]
                last_tracking_data = str(d["last_tracking_data"].strftime("%Y/%m/%d - %H:%M:%S %Z") if d["last_tracking_data"] is not None else NO)
                last_edited = d["last_edited"].strftime("%Y/%m/%d - %H:%M:%S %Z")
                code = (d["code"] or "unbekannt")
                ingame = (YES if d["state"]["ingame"] else NO)
                inmeeting = (YES if d["state"]["meeting"] else NO)

                p_alive = []
                p_dead = []
                p_unknown = []

                for c, i in d["players"].items():
                    if i["userid"]:
                        try:
                            member = self.bot.get_user(int(i["userid"])) or await self.bot.fetch_user(int(i["userid"]))
                            mention = member.mention
                        except NotFound:
                            mention = None
                    else:
                        mention = None

                    description = AMONGUS_PLAYER_COLORS[c][2] + " " + c.upper() + " "
                    description += ("- " + (mention if mention else "") + (" ('"+i["name"]+"')" if i["name"] else "")) if mention or i["name"] else ""

                    if i["exists"]:
                        if i["alive"]:
                            p_alive.append("- "+description)
                        else:
                            p_dead.append("- "+description)
                    else:
                        p_unknown.append("- "+description)

                t_alive = "\n".join(p_alive)
                t_dead = "\n".join(p_dead)
                t_unknown = "\n".join(p_unknown)

                embed = self.bot.getEmbed(
                    title=f"AmongUs Spiel (ID: { id })",
                    description=(f"""Infos über dieses Spiel

                        Letzte Trackerdaten - { last_tracking_data }  
                        Letzte Aktualisierung - { last_edited }

                        Code - { code }                  

                        In game - { ingame }          
                        In meeting - { inmeeting }

                        Spieler:
                    \u200b"""),
                    color=0xFEDE29,
                    inline=False,
                    fields=[
                        (f"{ALIVE} Lebend",      
                         '\u200b\n'+t_alive+'\n\u200b'),
                        (f"{DEAD} Tot",
                         '\u200b\n'+t_dead+'\n\u200b'),
                        (f"{UNKNOWN} Unbekannt",  
                         '\u200b\n'+t_unknown+'\n\u200b'),
                    ]
                )
                await msg.edit(embed=embed)
            except Exception as e:
                log('Error:', e)

    @amongus_backgroundtasks.before_loop
    async def amongus_backgroundtasks_before(self):
        print('[AmongUs Background Tasks] - Waiting for bot to be ready...')
        await self.bot.wait_until_ready()
        print('[AmongUs Background Tasks] - Started!')

    ## AmongUs Commands

    @commands.group(
        brief="Hauptcommand für alle AmongUs Befehle",
        description='Mute und entmute dich und deine Freunde automatisch während eines AmongUs-Spiels.',
        aliases=['au'],
        usage="<Unterbefehl> [Argumente]"
    )
    @commands.guild_only()
    async def amongus(self, ctx):
        if ctx.invoked_subcommand is None:
            await ctx.send_help()

    @amongus.command(
        name="create",
        brief="Erstelle ein AmongUs Spiel",
        aliases=['open', 'add', '+'],
    )
    @commands.guild_only()
    async def amongus_create(self, ctx):
        if await ctx.database.hasAmongUsGame():
            game = await ctx.database.getAmongUsGame()
            textchannel = ctx.guild.get_channel(int(game.text_channel_id))
            voicechannel = ctx.guild.get_channel(int(game.voice_channel_id))
            if textchannel and voicechannel:
                raise commands.BadArgument(message=f"Du hast bereits ein AmongUs-Spiel auf diesem Server! (ID { game.pk } - { textchannel.mention })")
            elif textchannel:
                raise commands.BadArgument(message=f"Du hast bereits ein AmongUs-Spiel auf diesem Server, jedoch konnte der Sprachkanal nicht mehr gefunden werden! Versuche dein Game mit `/au delete` zu löschen. (ID { game.pk } - { textchannel.mention })")
            else:
                raise commands.BadArgument(message=f"Du hast bereits ein AmongUs-Spiel auf diesem Server, jedoch konnte der Textkanal nicht mehr gefunden werden! (ID { game.pk })")
        else:
            category = await getAmongUsCategory(ctx.guild)
            textchannel = await category.create_text_channel(name=f"au-{ctx.author.name}#{ctx.author.discriminator}", reason="Benutzer hat AmongUs-Spiel erstellt.", topic=f"AmongUs Spiel von {ctx.author.name}#{ctx.author.discriminator}")
            voicechannel = await category.create_voice_channel(name=f"[AU] - {ctx.author.name}#{ctx.author.discriminator}", reason="Benutzer hat AmongUs-Spiel erstellt.")
            game = await ctx.database.createAmongUsGame(text_channel_id=str(textchannel.id), voice_channel_id=str(voicechannel.id))

            embed = ctx.getEmbed(
                title=f"AmongUs Spiel erstellt", 
                description=AMONGUS_TRACKER_INSTALL,
                color=0xFEDE29,
                fields=[
                    ("Url", str(game.get_tracker_url())),
                    ("ID", str(game.pk)),
                    ("API-Key", str(game.api_key)),
                    ("Kanal", str(textchannel.mention))
                ]
            )
            await ctx.author.send(embed=embed)

    @amongus.command(
        name="close",
        brief="Lösche dein AmongUs Spiel",
        aliases=['delete', 'del', '-'],
    )
    @commands.guild_only()
    async def amongus_close(self, ctx):
        if await ctx.database.hasAmongUsGame():
            game = await ctx.database.getAmongUsGame()
            textchannel = ctx.guild.get_channel(int(game.text_channel_id))
            voicechannel = ctx.guild.get_channel(int(game.voice_channel_id))
            if textchannel is not None:
                await textchannel.delete(reason="AmongUs Spiel wurde gelöscht!")
            if voicechannel is not None:
                await voicechannel.delete(reason="AmongUs Spiel wurde gelöscht!")

            id = str(game.pk)
            await ctx.database._delete(game)

            embed = ctx.getEmbed(
                title=f"AmongUs Spiel gelöscht! ({ id })",
                description="Dein AmongUs Spiel wurde erfolgreich gelöscht!",
                color=0xFEDE29,
            )
            await ctx.author.send(embed=embed)
        else:
            raise commands.BadArgument(message="Du hast kein AmongUs-Spiel auf diesem Server!")

    @amongus.command(
        name="reset",
        brief="Setze dein AmongUs Spiel zurück",
        aliases=[],
    )
    @commands.guild_only()
    async def amongus_reset(self, ctx):
        if await ctx.database.hasAmongUsGame():
            game = await ctx.database.getAmongUsGame()
            voicechannel = ctx.guild.get_channel(int(game.voice_channel_id))

            if voicechannel is not None:
                game.reset()
                await ctx.database._save(game)
                await voicechannel.edit(sync_permissions=True)
                await ctx.sendEmbed(
                    title="AmongUs Spiel zurückgesetzt!",
                    color=0xFEDE29,
                    description="Dein AmongUs Spiel wurde erfolgreich zurückgesetzt!"
                )
            else:
                raise commands.BadArgument(message="Der Sprachkanal zu deinem Spiel wurde nicht gefunden. Versuche dein Spiel mit `/amongus close` zu löschen")
        else:
            raise commands.BadArgument(message="Du hast kein AmongUs-Spiel auf diesem Server!")

    @amongus.command(
        name="apikey",
        brief="Erhalte nochmals die Konfigurationsdaten für den Tracker",
        aliases=['resend', 'config', 'tracker', 'key'],
    )
    @commands.guild_only()
    async def amongus_apikey(self, ctx):
        if await ctx.database.hasAmongUsGame():
            game = await ctx.database.getAmongUsGame()

            embed = ctx.getEmbed(
                title=f"AmongUs Tracker Konfiguration",
                description=AMONGUS_TRACKER_INSTALL,
                color=0xFEDE29,
                fields=[
                    ("Url", str(game.get_tracker_url())),
                    ("ID", str(game.pk)),
                    ("API-Key", str(game.api_key)),
                ]
            )
            await ctx.author.send(embed=embed)
        else:
            raise commands.BadArgument("Du hast kein AmongUs Spiel!")


    # VierGewinnt

    @commands.group(
        brief="Hauptcommand für alle VierGewinnt Befehle",
        description='Spiele Vier Gewinnt direkt in Discord.',
        aliases=['fourinarow', '4gewinnt', '4inarow', '4row', '4win'],
        help="Um eine Liste aller VierGewinnt-Befehle zu erhalten, gib den Command ohne Argumente ein.",
        usage="<Unterbefehl> [Argumente]"
    )
    async def viergewinnt(self, ctx):
        if ctx.invoked_subcommand is None:
            await ctx.send_help()

    @viergewinnt.command(
        name="duell",
        brief="Duelliere einen anderen Spieler",
        aliases=['battle', 'duel'],
        usage="<Mitglied> [Breite (4-10)] [Höhe (4-14)]",
    )
    @commands.guild_only()
    async def viergewinnt_duell(self, ctx, user: typing.Union[User, Member], width: int = 7, height: int = 6):
        if not (user == ctx.author or user.bot):
            msg = await ctx.sendEmbed(
                title="Vier Gewinnt",
                color=0x0078D7,
                description=f"Duell gegen {user.mention} wird erstellt..."
            )

            game = await ctx.database._create(VierGewinntGame, channel_id=str(ctx.channel.id), message_id=str(msg.id), player_1_id=str(ctx.author.id), player_2_id=str(user.id), width=width, height=height)

            embed = ctx.bot.getEmbed(
                title=f"Vier Gewinnt (#{game.pk})",
                color=0x0078D7,
                description=game.get_description()
            )

            await msg.edit(embed=embed)

            for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
                await msg.add_reaction(emoji)
        else:
            raise commands.BadArgument("Du kannst nicht gegen dich selbst oder Bots spielen... Wenn du einen Bot herausfordern möchtest, benutze bitte `/viergewinnt challenge`!")

    @viergewinnt.command(
        name="challenge",
        brief="Duelliere einen Bot",
        aliases=['bot', 'botduel', 'botduell'],
        usage="[Breite (4-10)] [Höhe (4-14)]",
    )
    async def viergewinnt_challenge(self, ctx, width: int = 7, height: int = 6):
        msg = await ctx.sendEmbed(
            title="Vier Gewinnt",
            color=0x0078D7,
            description=f"Challenge gegen einen Bot wird erstellt..."
        )

        game = await ctx.database._create(VierGewinntGame, channel_id=str(ctx.channel.id), message_id=str(msg.id), player_1_id=str(ctx.author.id), player_2_id=None, width=width, height=height)

        embed = ctx.bot.getEmbed(
            title=f"Vier Gewinnt (#{game.pk})",
            color=0x0078D7,
            description=game.get_description()
        )

        await msg.edit(embed=embed)

        for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
            await msg.add_reaction(emoji)

    @viergewinnt.command(
        name="games",
        brief="Liste alle deine Spiele auf",
        aliases=["list"],
        usage="[User]",
    )
    async def viergewinnt_games(self, ctx, user: typing.Union[User, Member] = None):
        user = user or ctx.author
        challenges = await ctx.database._list(VierGewinntGame, _order_by=("-time_created",), player_1_id=str(user.id), player_2_id__isnull=True)
        duels_created = await ctx.database._list(VierGewinntGame, _order_by=("-time_created",), player_1_id=str(user.id), player_2_id__isnull=False)
        duels_invited = await ctx.database._list(VierGewinntGame, _order_by=("-time_created",), player_2_id=str(user.id))
        challengetext = "" if challenges else "Du hast noch keine Challenges erstellt."
        for g in challenges:
            challengetext += ((UNKNOWN if g.winner_id is None else YES if g.winner_id == str(user.id) else NO)
                              if g.finished else RUNNING) + f" ({ g.id }) BOT " + g.time_created.strftime("%Y/%m/%d - %H:%M %Z") + "\n"
        duelscreatedtext = "" if duels_created else "Du hast noch keine Duelle erstellt."
        for g in duels_created:
            duelscreatedtext += ((UNKNOWN if g.winner_id is None else YES if g.winner_id == str(user.id) else NO) 
                                 if g.finished else RUNNING) + f" ({ g.id }) <@{ g.player_2_id }> " + g.time_created.strftime("%Y/%m/%d - %H:%M %Z") + "\n"
        duelsinvitedtext = "" if duels_invited else "Du wurdest noch nicht zu einem Duell eingeladen."
        for g in duels_invited:
            duelsinvitedtext += ((UNKNOWN if g.winner_id is None else YES if g.winner_id == str(user.id) else NO)
                                 if g.finished else RUNNING) + f" ({ g.id }) <@{ g.player_1_id }> " + g.time_created.strftime("%Y/%m/%d - %H:%M %Z") + "\n"

        await ctx.sendEmbed(
            title="VierGewinnt Spiele", 
            description="Vier Gewinnt Spiele von "+user.mention, 
            color=0x0078D7, 
            inline=False,
            fields=[
                ("Challenges", challengetext+"\u200b"), 
                ("Erstellte Spiele", duelscreatedtext+"\u200b"), 
                ("Eingeladene Spiele", duelsinvitedtext+"\u200b")
            ]
        )

    @viergewinnt.command(
        name="reset",
        brief="Lösche alle deine Challenges und Duelle",
        description="Hinweis: Dies beinhaltet nicht Duelle, welche ein anderer Spieler gestartet hat!",
        aliases=[],
        usage="[NUR BESITZER: Spieler]"
    )
    async def viergewinnt_reset(self, ctx, user: typing.Union[User, Member] = None):
        user = ctx.author if user is None else user if await self.bot.is_owner(user) else ctx.author
        await ctx.database._delete(await ctx.database._list(VierGewinntGame, get_as_queryset=True, player_1_id=str(user.id)))
        await ctx.sendEmbed(
            title="VierGewinnt Spiele gelöscht!",
            description=f"Die VierGewinnt Spiele von { user.mention } wurden erfolgreich gelöscht!",
            color=0x0078D7, 
        )

    @viergewinnt.command(
        name="resume",
        brief="Fahre ein Duell oder eine Challenge fort",
        description="Wenn du die Nachricht eines Duells oder einer Challenge nicht mehr finden kannst, kannst du sie mit diesem Befehl noch einmal senden lassen. Hiermit können auch Spiele von anderen Spielern angesehen werden.",
        aliases=["continue", "wiederaufnehmen", "resend", "view"],
        usage="<ID>",
        help="Um die ID herauszufinden, benutze `/viergewinnt games`",
    )
    async def viergewinnt_resume(self, ctx, id: int):
        if await ctx.database._has(VierGewinntGame, id=id):
            game = await ctx.database._get(VierGewinntGame, id=id)

            if str(ctx.author.id) in [game.player_1_id, game.player_2_id]:
                msg = await ctx.sendEmbed(
                    title=f"Vier Gewinnt (#{game.pk})",
                    color=0x0078D7,
                    description=game.get_description()
                )

                oldchannelid = int(game.channel_id)
                oldmessageid = int(game.message_id)

                game.channel_id = msg.channel.id
                game.message_id = msg.id

                await ctx.database._save(game)

                if not game.finished:
                    for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
                        await msg.add_reaction(emoji)

                    try:
                        channel = self.bot.get_channel(oldchannelid) or await self.bot.fetch_channel(oldchannelid)
                        message = await channel.fetch_message(oldmessageid)

                        await message.edit(embed=self.bot.getEmbed(
                            title=f"Vier Gewinnt (#{game.pk})",
                            color=0x0078D7,
                            description="Dieses Spiel wurde in einer neuen Nachricht weitergeführt!\n\n"+game.get_description()
                        ))

                        for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
                            await message.remove_reaction(emoji, self.bot.user)
                    except NotFound as e:
                        print(e)
            else:
                msg = await ctx.sendEmbed(
                    title=f"Vier Gewinnt (#{game.pk} - Readonly)",
                    color=0x0078D7,
                    description=game.get_description()
                )
        else:
            raise commands.BadArgument("Ein Spiel mit dieser ID konnte nicht gefunden werden! Möglicherweise gelöscht?")

def setup(bot):
    bot.add_cog(Games(bot))



import youtube_dl
import os
import asyncio

from discord import PCMVolumeTransformer, FFmpegPCMAudio
from discord.ext import commands

from discordbot.config import FILESPATH, FFMPEG_OPTIONS, FFMPEG_OPTIONS_STREAM, DEBUG

# Suppress noise about console usage from errors
youtube_dl.utils.bug_reports_message = lambda: ''

_ytdl = youtube_dl.YoutubeDL({
    'format': 'bestaudio/best',
    'outtmpl': os.path.join(FILESPATH, 'youtube', '%(extractor)s-%(id)s-%(title)s.%(ext)s'),
    'restrictfilenames': True,
    'noplaylist': True,
    'nocheckcertificate': True,
    'ignoreerrors': True,
    'logtostderr': False,
    'quiet': True,
    'no_warnings': True,
    'default_search': 'auto',
    'ffmpeg_location': FILESPATH,
    # bind to ipv4 since ipv6 addresses cause issues sometimes
    'source_address': '0.0.0.0'
})

class AudioManager():
    def __init__(self, ctx):
        self.guild = ctx.guild
        self.loop = ctx.bot.loop
        self.database = ctx.database
        self.bot = ctx.bot
        
    async def playlist(self):
        return await self.database.get_playlist()

    async def refetchUrlIfYouTube(self, watchurl:str):
        if "youtube.com" in watchurl:
            data = await self.loop.run_in_executor(None, lambda: _ytdl.extract_info(watchurl, download=False))
            return [f for f in (data["entries"][0]["formats"] if "entries" in data else data["formats"]) if not f["acodec"] is None][0]
        else:
            return watchurl

    async def getSongs(self, query: str):
        data = await self.loop.run_in_executor(None, lambda: _ytdl.extract_info(query, download=False))

        if data:
            if 'entries' in data:
                return [await self.database.getOrCreateAudioSourceFromDict(d) for d in data['entries'] if not d is None]
            return [await self.database.getOrCreateAudioSourceFromDict(data)]
        return []

    async def getInfoEmbedData(self, query: str):
        data = await self.loop.run_in_executor(None, lambda: _ytdl.extract_info(query, download=False))

        if data:
            if DEBUG:
                with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "_debug.test.json"), "w+") as f:
                    import json
                    f.write(json.dumps(data, indent=2))

            if "entries" in data:
                if len(data["entries"]) > 1:
                    entries = list(entry for entry in data["entries"] if not entry is None)
                    return {
                        "title": "Playlist",
                        "description": f"{len(entries)} Stücke",
                        "authorname": data.get("title", "Unbekannt"),
                        "authorurl": data["webpage_url"] if "webpage_url" in data and "http" in data["webpage_url"] else ""
                    }
                else:
                    data = data["entries"][0]
            src = await self.database.getOrCreateAudioSourceFromDict(data)
            return {
                "title": src.title,
                "authorname": src.uploader_name,
                "authorurl": src.uploader_url,
                "description": src.description,
                "thumbnailurl": src.url_thumbnail,
                "inline": True,
                "fields": [
                    ("Dauer", src.duration_calc),
                    ("Ansehen/Anhören", f"[Hier klicken]({src.url_watch})")
                ]
            }

    async def processQuery(self, query:str):
        sources = await self.getSongs(query)
        print(sources)



class YouTubePlayer(PCMVolumeTransformer):
    def __init__(self, filename, *, queue, data, stream=False, volume=0.5):
        source = FFmpegPCMAudio(filename, **(FFMPEG_OPTIONS_STREAM if stream else FFMPEG_OPTIONS))

        super().__init__(source, volume)

        self.queue = queue

        self.data = data

        self.url = data.get('url', '')
        self.link = data.get('webpage_url', self.url)
        self.title = data.get('title', 'Unbekannter Titel')

        self.uploader = data.get('uploader', "")
        self.uploader_url = data.get('uploader_url', "")
        self.thumbnail = data.get('thumbnail', "")
        self.description = data.get('description', "")
        self.duration = int(data.get('duration', 0))

    async def send(self, ctx, status: str = "Wird jetzt gespielt..."):
        fields = [("Ansehen/Anhören", "[Hier klicken]("+self.link+")")]
        if self.duration:
            fields.append(("Dauer", str(int(self.duration/60)) +
                           "min "+str(int(self.duration % 60))+"s"))
        if status:
            fields.append(("Status", status, False))
        await ctx.sendEmbed(
            title=self.title,
            description=((self.description if len(self.description) < 100 else self.description[0:100]+"...") if isinstance(
                self.description, str) else "Keine Beschreibung gefunden"),
            color=ctx.cog.color,
            fields=fields,
            thumbnailurl=self.thumbnail if self.thumbnail else None,
            authorname=self.uploader if self.uploader else None,
            authorurl=self.uploader_url if self.uploader_url else None,
        )

    def getinfo(self):
        return f"[{self.title}]({self.link})"

    def play(self, ctx):
        if ctx.voice_client:
            if ctx.voice_client.is_playing():
                ctx.voice_client.stop()
            ctx.voice_client.play(
                self, after=lambda e: self.queue.playNext(ctx))

    @classmethod
    async def from_url(self, url, *, queue, loop=None, stream=False):
        loop = loop or asyncio.get_event_loop()
        data = await loop.run_in_executor(None, lambda: _ytdl.extract_info(url, download=not stream))

        if 'entries' in data:
            data = data['entries'][0]

        filename = data['url'] if stream else _ytdl.prepare_filename(data)
        return self(queue=queue, filename=filename, data=data, stream=stream)



# Generated by Django 3.1.2 on 2020-10-23 19:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0007_auto_20201023_2145'),
    ]

    operations = [
        migrations.AddField(
            model_name='amongusgame',
            name='last_tracking_data',
            field=models.DateTimeField(default=None, null=True, verbose_name='Last tracking data'),
        ),
    ]



from discord.ext import commands
from discord import User, TextChannel, utils
from datetime import datetime
import typing

from discordbot.config import INVITE_OWNER, INVITE_BOT, SPAM_ALLOWED
#from discordbot.errors import ErrorMessage

REGELN = {
    "1) Verhalten":
        [
            "Sei nett zu anderen Leuten und behandle sie so, wie auch du behandelt werden möchtest!", 
            "Beleidige keine anderen Leute!",
            "Benutze anständige Sprache!",
            "Bleib beim Thema!",
        ],
    "2) Text":
        [
            "Spamming ist verboten!", 
            "Werbung ohne Erlaubnis eines Administrators ist verboten!",
        ],
    "3) Ton":
        [
            "Benutze keinen Stimmverzerrer!", 
            "Mache keine unnötigen Hintergrundgeräusche!", 
            "Channel Hopping bitte unterlassen!", 
            "Sprachaufnahmen sind nur mit Erlaubnis aller Teilnehmer gestattet!",
        ],
    "4) NSFW":
        [
            "Anstössige Inhalte ausserhalb NSFW-Kanälen werden sofort gelöscht und der Autor mit einem Bann/Mute bestraft!", 
        ],
    "5) Sicherheit":
        [
            "Anweisungen von Moderatoren, Supportern und Admins müssen befolgt werden!",
            "Falls jemand ohne Grund nach deinen persönlichen Daten fragt, ignoriere bitte die Nachricht und meldet sie einem Admin oder dem Serverbesitzer! Bitte melde den Benutzer ebenfalls bei Discord!",
            "Sende nie jemandem euer Passwort!",
        ],
    "6) Empfehlungen":
        [
            "Habe Spass!",
        ],
}


class Basic(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.color = 0xffffff

    @commands.command(
        brief="Zeigt den Ping des Bots an",
        description='Gibt den aktuellen Ping zurück',
        aliases=['p'],
        help="Gib einfach /ping ein und warte ab.",
        usage=""
    )
    async def ping(self, ctx):
        start = datetime.timestamp(datetime.now())
        msg = await ctx.sendEmbed(title="Aktueller Ping", fields=[("Ping", "Berechnen...")])
        embed = ctx.getEmbed(title="Aktueller Ping", fields=[("Ping", str(int(( datetime.timestamp( datetime.now() ) - start ) * 1000))+"ms")])
        await msg.edit(embed=embed)
        return

    @commands.command(
        brief="Schreibt dir nach",
        description="Gibt den angegebenen Text zurück",
        aliases=["copy"],
        help="Benutze /say <Text> und der Bot schickt dir den Text zurück",
        usage="<Text>"
        )
    async def say(self, ctx, text:str, *args):
        txt = " ".join((text,)+args)
        await ctx.send(txt)

    @commands.command(
        brief="Siehe den Avatar eines Benutzers",
        description="Erhalte den Standardavatar und Avatar eines Benutzers",
        aliases=["defaultavatar"],
        help="Benutze /avatar <User> und du siehst, welchen Avatar die Person hat",
        usage="<User>"
        )
    async def avatar(self, ctx, user:User):
        await ctx.sendEmbed(title="Avatar", fields=[("Benutzer",user.mention),("Standardavatar",user.default_avatar)], thumbnailurl=str(user.avatar_url))


    @commands.command(
        brief="Spamt jemanden voll",
        description="Schickt jemandem ein paar Nachrichten",
        aliases=["troll"],
        help="Benutze /spam <User> und der Bot spamt den User voll",
        usage="<Kanal/Benutzer> [Anzahl<=10] [Text]"
        )
    async def spam(self, ctx, what:typing.Union[TextChannel,User]=None, anzahl:int=5, *args):
        if SPAM_ALLOWED:
            anzahl = int(anzahl if anzahl <= 10 else 10)
            text = str(" ".join(str(i) for i in args))
            empty = not (len(text) > 0 and not text == (" "*len(text)))
            for _ in range(anzahl):
                if not empty:
                    await what.send(text+" von "+ctx.author.mention)
                else:
                    await what.send("Spam"+" von "+ctx.author.mention)
            return
        else:
            await ctx.invoke(ctx.bot.get_command("regeln"))
            await ctx.sendEmbed(
                title="Spam verboten",
                description="Schon vergessen, dass Spam laut Regeln verboten ist? Hast du wirklich gedacht, ich breche meine eigenen Regeln?"
            )
            if ctx.guild is not None:
                await ctx.database.createReport(dc_user=ctx.author, reason="Tried to use /spam", reportedby_dc_user=ctx.guild.me)


    @commands.command(
        brief="Zeigt die Regeln",
        description="Schickt die Regeln in den Chat",
        aliases=["rules"],
        help="Benutze /regeln um dich oder jemand anderes daran zu erinnern!",
        usage="<Kanal/Benutzer> [Anzahl<100] [Text]"
        )
    async def regeln(self,ctx):
        EMBED = self.bot.getEmbed(title="Regeln", description="Das Nichtbeachten der Regeln kann mit einem Ban, Kick oder Mute bestraft werden!")
        owner = self.bot.get_user(self.bot.owner_id)
        if owner:
            EMBED.set_footer(text=f'Besitzer dieses Bots ist {owner.name}#{owner.discriminator}',icon_url=owner.avatar_url)
        for kategorie in REGELN:
            EMBED.add_field(name=kategorie,value="```nimrod\n- "+ ("\n- ".join(regel for regel in REGELN[kategorie])) +"```",inline=False)
        await ctx.send(embed=EMBED)


    @commands.command(
        brief="Erhalte eine Einladung",
        description="Schickt dir eine Einladung zum Server und Bot",
        aliases=["invitelink"],
        help="Benutze /invite und erhalte eine Einladung zu diesem Server, dem Bot-Server und einen Link, um den Bot zum eigenen Server hinzuzufügen",
        usage=""
        )
    async def invite(self,ctx):
        invite = None
        if ctx.guild:
            try:
                invite = await ctx.guild.vanity_invite()
            except:
                try:
                    invite = utils.get(list(await ctx.guild.invites()), max_age=0, max_uses=0)
                    if not invite:
                        invite = await ctx.channel.create_invite()
                except:
                    invite = None

        if invite:
            await ctx.sendEmbed(title="Einladungen", fields=[("Dieser Server", invite.url or "Unbekannt"), ("Bot Owner Server", INVITE_OWNER), ("Bot", INVITE_BOT)])
        else:
            await ctx.sendEmbed(title="Einladungen", fields=[("Bot Owner Server",INVITE_OWNER), ("Bot",INVITE_BOT)])

def setup(bot):
    bot.add_cog(Basic(bot))



# Generated by Django 3.1.3 on 2020-11-08 15:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0011_auto_20201031_1721'),
    ]

    operations = [
        migrations.CreateModel(
            name='BotPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('id_1', models.CharField(max_length=32, verbose_name='ID 1')),
                ('id_2', models.CharField(max_length=32, verbose_name='ID 2')),
                ('typ', models.CharField(max_length=32, verbose_name='Typ')),
            ],
        ),
    ]
